@@ -699,6 +699,11 @@ public final class FreeColClient {
     public void addSpecificationActions(Specification specification) {
         SwingUtilities.invokeLater(() -> {
             actionManager.addSpecificationActions(specification);
+            // XXX: The actions are loaded asynchronously without a callback.
+            SwingUtilities.invokeLater(() -> {
+                getGUI().resetMenuBar();
+                getGUI().resetMapControls();
+            });
         });
     }
 
@@ -989,8 +994,8 @@ public final class FreeColClient {
         if (gui.confirm("retireDialog.areYouSure.text", "ok", "cancel")) {
             final Player player = getMyPlayer();
             player.changePlayerType(Player.PlayerType.RETIRED);
-            getInGameController().highScore(null);
-            askServer().retire();
+            final boolean highScore = askServer().retire();
+            getInGameController().highScore(highScore);
         }
     }
 
