@@ -981,10 +981,10 @@ public class GUI extends FreeColClientHolder {
      * Show a save file dialog, selecting one to load.
      *
      * @param root The root directory to look in.
-     * @param extension The file extension to look for.
+     * @param extension The file extensions to look for.
      * @return The {@code File} selected, or null on error.
      */
-    public final File showLoadSaveFileDialog(File root, String extension) {
+    public final File showLoadSaveFileDialog(File root, String... extension) {
         File file = showLoadDialog(root, extension);
         if (file != null && !file.isFile()) {
             showErrorPanel(FreeCol.badFile("error.noSuchFile", file));
@@ -1429,6 +1429,18 @@ public class GUI extends FreeColClientHolder {
      */
     public void closeMenus() {}
 
+    /**
+     * Resets the map controls in order to properly reference any
+     * newly recreated action.
+     */
+    public void resetMapControls() {}
+    
+    /**
+     * Resets the menu bar in order to properly reference any
+     * newly recreated action.
+     */
+    public void resetMenuBar() {}
+    
     /**
      * Update the menu bar.
      *
@@ -1574,6 +1586,21 @@ public class GUI extends FreeColClientHolder {
      */
     public void changeView() {}
 
+    /**
+     * Sets if ranged attack mode should be activated.
+     * 
+     * @param rangedAttackMode If {@code true}, then display tiles reachable
+     *      by ranged attack and allow the attack to be performed by clicking. 
+     */
+    public void setRangedAttackMode(boolean rangedAttackMode) {}
+    
+    /**
+     * Toggles if ranged attack mode should be activated.
+     * 
+     * @see #setRangedAttackMode(boolean)
+     */
+    public void toggleRangedAttackMode() {}
+    
 
     // Zoom controls
 
@@ -2060,7 +2087,7 @@ public class GUI extends FreeColClientHolder {
      * @param extension An extension to select with.
      * @return The selected {@code File}.
      */
-    public File showLoadDialog(File directory, String extension) { return null; }
+    public File showLoadDialog(File directory, String... extension) { return null; }
 
     /**
      * Show the LoadingSavegameDialog.
@@ -2506,4 +2533,14 @@ public class GUI extends FreeColClientHolder {
     public boolean canGameChangingModsBeAdded() {
         return true;
     }
+
+
+    /**
+     * A method to be called only on serious Errors in order
+     * to ensure sufficient memory for displaying an error
+     * message.
+     */
+    public void emergencyPurge() {
+
+    }
 }
