@@ -489,6 +489,10 @@ public final class ImageLibrary {
     public BufferedImage getScaledImage(String key) {
         return this.imageCache.getScaledImage(key, this.scaleFactor, false);
     }
+    
+    public BufferedImage getRangedTargetCrosshair() {
+        return getScaledImage("image.rangedTarget");
+    }
 
     /**
      * Get the image for the given identifier, using a given size.
@@ -1083,6 +1087,15 @@ public final class ImageLibrary {
                                               TileImprovementStyle riverStyle) {
         return getForestImageInternal(type, riverStyle, this.tileForestSize);
     }
+    
+    public BufferedImage getForestCornerImage(TileType type, Direction corner) {
+        final String key = "image.tileforest." + type.getId() + "." + corner.toString().toLowerCase();
+        final ImageResource ir = ResourceManager.getImageResource(key, false);
+        if (ir == null) {
+            return null;
+        }
+        return imageCache.getSizedImage(key, this.tileForestSize, false, 0);
+    }
 
 
     /**
@@ -1301,6 +1314,10 @@ public final class ImageLibrary {
                 String stockade = colony.getStockadeKey();
                 if (stockade != null) key += "." + stockade;
             }
+            final String extraKey = key + "." + settlement.getOwner().getNationResourceKey();
+            if (ResourceManager.getImageResource(extraKey, false) != null) {
+                key = extraKey;
+            }
         } else if (settlement instanceof IndianSettlement) {
             IndianSettlement is = (IndianSettlement)settlement;
             if (is.hasMissionary()) key += ".mission";
