@@ -65,6 +65,7 @@ import net.sf.freecol.client.gui.dialog.FreeColDialog;
 import net.sf.freecol.client.gui.dialog.Parameters;
 import net.sf.freecol.client.gui.mapviewer.GUIMessage;
 import net.sf.freecol.client.gui.mapviewer.MapViewer;
+import net.sf.freecol.client.gui.mapviewer.MapViewerState;
 import net.sf.freecol.client.gui.mapviewer.TileViewer;
 import net.sf.freecol.client.gui.panel.ColonyPanel;
 import net.sf.freecol.client.gui.panel.FreeColImageBorder;
@@ -133,6 +134,11 @@ public class SwingGUI extends GUI {
         CENTERED,
         CENTERED_LEFT,
         CENTERED_RIGHT,
+        
+        /**
+         * Places centered even this means overlapping components.
+         */
+        CENTERED_FORCED
     }
 
     /** European subpanel classes. */
@@ -417,6 +423,25 @@ public class SwingGUI extends GUI {
         clearGotoPath();
         return true;
     }
+    
+    /**
+     * {@inheritDoc}
+     */
+    public void setRangedAttackMode(boolean rangedAttackMode) {
+        final MapViewerState mvs = this.mapViewer.getMapViewerState();
+        if (mvs.isRangedAttackMode() == rangedAttackMode) {
+            return;
+        }
+        mvs.setRangedAttackMode(rangedAttackMode);
+        refresh();
+    }
+    
+    /**
+     * {@inheritDoc}
+     */
+    public void toggleRangedAttackMode() {
+        setRangedAttackMode(!this.mapViewer.getMapViewerState().isRangedAttackMode());
+    }
 
     /**
      * Change the selected tile.
@@ -716,8 +741,6 @@ public class SwingGUI extends GUI {
     public void reconnectGUI(Unit active, Tile tile) {
         this.canvas.requestFocusInWindow();
         this.canvas.initializeInGame();
-        enableMapControls(getClientOptions()
-            .getBoolean(ClientOptions.DISPLAY_MAP_CONTROLS));
         closeMenus();
         clearGotoPath();
         this.canvas.resetMenuBar();
@@ -738,6 +761,10 @@ public class SwingGUI extends GUI {
             changeView((Unit)null, false);
         }
         this.mapViewer.getMapViewerBounds().setFocus(tile);
+
+        enableMapControls(false);
+        enableMapControls(getClientOptions().getBoolean(ClientOptions.DISPLAY_MAP_CONTROLS));
+
         refresh();
     }
         
@@ -950,11 +977,11 @@ public class SwingGUI extends GUI {
     public void enableMapControls(boolean enable) {
         if (this.mapControls == null) return;
         if (enable) {
+            updateMapControls();
             this.canvas.addMapControls();
         } else {
             this.canvas.removeMapControls();
         }
-        updateMapControls();
     }
 
     /**
@@ -1016,6 +1043,14 @@ public class SwingGUI extends GUI {
         this.canvas.closeMenus();
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public void resetMenuBar() {
+        this.canvas.resetMenuBar();
+    }
+    
     /**
      * {@inheritDoc}
      */
@@ -1394,6 +1429,20 @@ public class SwingGUI extends GUI {
      */
     @Override
     public void clickAt(int count, int x, int y) {
+        if (mapViewer.getMapViewerState().isRangedAttackMode()) {
+            final Tile tile = tileAt(x, y);
+            final Unit activeUnit = mapViewer.getMapViewerState().getActiveUnit();
+            if (activeUnit == null
+                    || !activeUnit.isOffensiveUnit()
+                    || !activeUnit.canAttackRanged(tile)
+                    || activeUnit.getMovesLeft() <= 0
+                    || !getFreeColClient().currentPlayerIsMyPlayer()) {
+                return;
+            }
+            igc().attackRanged(activeUnit, tile);
+            return;
+        }
+        
         // This could be a drag, which would have already been processed
         // in @see CanvasMouseListener#mouseReleased
         if (count == 1 && isDrag(x, y)) return;
@@ -1748,15 +1797,23 @@ public class SwingGUI extends GUI {
         }
         if (this.canvas != null) { 
             this.canvas.resetMenuBar();
-            if (this.canvas.removeMapControls()) {
-                this.canvas.addMapControls();
-            }
+            resetMapControls();
         }
         
-        updateMapControls();
         refresh();
     }
 
+
+    /**
+     * {@inheritDoc}
+     */
+    public void resetMapControls() {
+        if (this.canvas.removeMapControls()) {
+            updateMapControls();
+            this.canvas.addMapControls();
+        }
+    }
+
     private int determineMainFontSizeUsingClientOptions(final int dpi) {
         final int DEFAULT_DPI = 72;
         
@@ -2056,10 +2113,11 @@ public class SwingGUI extends GUI {
      * {@inheritDoc}
      */
     @Override
-    public File showLoadDialog(File directory, String extension) {
-        FileFilter[] filters = new FileFilter[] {
-            FreeColDataFile.getFileFilter(extension)
-        };
+    public File showLoadDialog(File directory, String... extension) {
+        final FileFilter[] filters = new FileFilter[extension.length];
+        for (int i=0; i<extension.length; i++) {
+            filters[i] = FreeColDataFile.getFileFilter(extension[i]);
+        }
         File file = null;
         for (;;) {
             file = this.widgets.showLoadDialog(directory, filters);
@@ -2149,7 +2207,7 @@ public class SwingGUI extends GUI {
         InformationPanel panel
             = new InformationPanel(getFreeColClient(), texts, fcos, icons);
         return this.canvas.showFreeColPanel(panel,
-            getPopupPosition(tile), true);
+            getPopupPosition(tile), false);
     }
 
     /**
@@ -2408,7 +2466,8 @@ public class SwingGUI extends GUI {
     public File showSaveDialog(File directory, String defaultName) {
         String extension = lastPart(defaultName, ".");
         FileFilter[] filters = new FileFilter[] {
-            FreeColDataFile.getFileFilter(extension)
+            FreeColDataFile.getFileFilter(extension),
+            FreeColDataFile.getFileFilter("*")
         };
         return this.widgets.showSaveDialog(directory, filters, defaultName);
     }
