@@ -30,6 +30,7 @@ import net.sf.freecol.client.control.InGameController;
 import net.sf.freecol.client.gui.action.ColopediaAction.PanelType;
 import net.sf.freecol.client.gui.action.DisplayTileTextAction.DisplayText;
 import net.sf.freecol.client.gui.panel.UnitButton;
+import net.sf.freecol.common.model.Ability;
 import net.sf.freecol.common.model.Direction;
 import net.sf.freecol.common.model.Specification;
 import net.sf.freecol.common.model.TileImprovementType;
@@ -136,6 +137,7 @@ public class ActionManager extends OptionGroup {
         add(new PreferencesAction(freeColClient));
         add(new SaveAndQuitAction(freeColClient));
         add(new QuitAction(freeColClient));
+        add(new AttackRangedAction(freeColClient));
         add(new ReconnectAction(freeColClient));
         add(new RenameAction(freeColClient));
         add(new ReportCargoAction(freeColClient));
@@ -183,6 +185,11 @@ public class ActionManager extends OptionGroup {
      */
     public void addSpecificationActions(Specification spec) {
         // Initialize ImprovementActions
+        for (Option<?> o : new ArrayList<>(getOptions())) {
+            if (o instanceof ImprovementAction) {
+                remove(o.getId());
+            }
+        }
         for (TileImprovementType type : spec.getTileImprovementTypeList()) {
             if (!type.isNatural()) {
                 add(new ImprovementAction(freeColClient, type));
@@ -236,6 +243,9 @@ public class ActionManager extends OptionGroup {
      */
     public List<UnitButton> makeUnitActionButtons(final Specification spec) {
         List<UnitButton> ret = new ArrayList<>();
+        if (spec == null || spec.hasAbility(Ability.HITPOINTS_COMBAT_MODEL)) {
+            ret.add(new UnitButton(this, AttackRangedAction.id));
+        }
         ret.add(new UnitButton(this, WaitAction.id));
         ret.add(new UnitButton(this, SkipUnitAction.id));
         ret.add(new UnitButton(this, SentryAction.id));
