@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015   The FreeCol Team
+ *  Copyright (C) 2002-2022   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -33,9 +33,9 @@ public class ShowGameOptionsAction extends FreeColAction {
 
 
     /**
-     * Creates a new <code>ShowGameOptionsAction</code>.
+     * Creates a new {@code ShowGameOptionsAction}.
      *
-     * @param freeColClient The <code>FreeColClient</code> for the game.
+     * @param freeColClient The {@code FreeColClient} for the game.
      */
     public ShowGameOptionsAction(FreeColClient freeColClient) {
         super(freeColClient, id);
@@ -49,6 +49,6 @@ public class ShowGameOptionsAction extends FreeColAction {
      */
     @Override
     public void actionPerformed(ActionEvent ae) {
-        getGUI().showGameOptionsDialog(false, false);
+        getGUI().showGameOptionsDialog(false, null);
     }
 }
