@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015   The FreeCol Team
+ *  Copyright (C) 2002-2022   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -25,7 +25,7 @@ import net.sf.freecol.client.FreeColClient;
 
 
 /**
- * An action for displaying a Tile Popup.
+ * An action for displaying a tile popup.
  */
 public class TilePopupAction extends MapboardAction {
 
@@ -33,9 +33,9 @@ public class TilePopupAction extends MapboardAction {
 
 
     /**
-     * Creates a new <code>TilePopupAction</code>.
+     * Creates a new {@code TilePopupAction}.
      *
-     * @param freeColClient The <code>FreeColClient</code> for the game.
+     * @param freeColClient The {@code FreeColClient} for the game.
      */
     public TilePopupAction(FreeColClient freeColClient) {
         super(freeColClient, id);
@@ -49,6 +49,6 @@ public class TilePopupAction extends MapboardAction {
      */
     @Override
     public void actionPerformed(ActionEvent ae) {
-        getGUI().showTilePopUpAtSelectedTile();
+        getGUI().showTilePopup(getGUI().getSelectedTile());
     }
 }
