@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015   The FreeCol Team
+ *  Copyright (C) 2002-2022   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -22,6 +22,7 @@ package net.sf.freecol.client.gui.action;
 import java.awt.event.ActionEvent;
 
 import net.sf.freecol.client.FreeColClient;
+import net.sf.freecol.client.gui.GUI;
 
 
 /**
@@ -36,7 +37,7 @@ public class ToggleViewModeAction extends MapboardAction {
     /**
      * Creates this action.
      *
-     * @param freeColClient The <code>FreeColClient</code> for the game.
+     * @param freeColClient The {@code FreeColClient} for the game.
      */
     public ToggleViewModeAction(FreeColClient freeColClient) {
         super(freeColClient, id);
@@ -50,6 +51,18 @@ public class ToggleViewModeAction extends MapboardAction {
      */
     @Override
     public void actionPerformed(ActionEvent ae) {
-        getGUI().toggleViewMode();
+        // Only really toggles between terrain and units modes.
+        final GUI gui = getGUI();
+        GUI.ViewMode vm = gui.getViewMode();
+        switch (vm) {
+        case MOVE_UNITS:
+            gui.changeView(gui.getSelectedTile());
+            break;
+        case TERRAIN:
+            gui.changeView(gui.getActiveUnit(), false);
+            break;
+        default:
+            break;
+        }
     }
 }
