@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015   The FreeCol Team
+ *  Copyright (C) 2002-2022   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -26,10 +26,10 @@ import java.awt.event.ActionListener;
 import java.awt.event.MouseAdapter;
 import java.awt.event.MouseEvent;
 
-import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.function.Predicate;
 
 import javax.swing.AbstractCellEditor;
 import javax.swing.DefaultCellEditor;
@@ -50,6 +50,7 @@ import javax.swing.table.TableColumnModel;
 
 import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.control.PreGameController;
+import net.sf.freecol.client.gui.FontLibrary;
 import net.sf.freecol.client.gui.GUI;
 import net.sf.freecol.client.gui.action.ColopediaAction.PanelType;
 import net.sf.freecol.client.gui.plaf.FreeColComboBoxRenderer;
@@ -63,6 +64,7 @@ import net.sf.freecol.common.model.NationType;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Specification;
 import net.sf.freecol.common.model.StringTemplate;
+import static net.sf.freecol.common.util.CollectionUtils.*;
 
 
 /**
@@ -73,14 +75,14 @@ public final class PlayersTable extends JTable {
     /**
      * A table cell editor that can be used to select a nation.
      */
-    private class AdvantageCellEditor extends DefaultCellEditor {
+    private static class AdvantageCellEditor extends DefaultCellEditor {
 
         private final JComboBox<EuropeanNationType> box;
 
         /**
          * Internal constructor.
          *
-         * @param box The <code>JComboBox</code> to edit.
+         * @param box The {@code JComboBox} to edit.
          */
         private AdvantageCellEditor(JComboBox<EuropeanNationType> box) {
             super(box);
@@ -91,7 +93,7 @@ public final class PlayersTable extends JTable {
         /**
          * A standard constructor.
          *
-         * @param nationTypes List of <code>EuropeanNationType></code>
+         * @param nationTypes The list of {@code EuropeanNationType}s.
          */
         public AdvantageCellEditor(List<EuropeanNationType> nationTypes) {
             this(new JComboBox<>(nationTypes
@@ -112,7 +114,7 @@ public final class PlayersTable extends JTable {
         }
     }
 
-    private class AdvantageCellRenderer extends JLabel
+    private static class AdvantageCellRenderer extends JLabel
         implements TableCellRenderer {
 
         /** The national advantages type. */
@@ -122,7 +124,7 @@ public final class PlayersTable extends JTable {
         /**
          * The default constructor.
          *
-         * @param advantages The type of national <code>Advantages</code>.
+         * @param advantages The type of national {@code Advantages}.
          */
         public AdvantageCellRenderer(Advantages advantages) {
             this.advantages = advantages;
@@ -143,11 +145,12 @@ public final class PlayersTable extends JTable {
             final NationType nationType = ((Nation)table.getValueAt(row,
                     PlayersTable.NATION_COLUMN)).getType();
             JLabel label;
-            switch (advantages) {
+            switch (this.advantages) {
             case SELECTABLE:
-                return Utility.localizedLabel(Messages.nameKey((player == null)
+                label = Utility.localizedLabel(Messages.nameKey((player == null)
                         ? nationType
                         : player.getNationType()));
+                break;
             case FIXED:
                 label = Utility.localizedLabel(Messages.nameKey(nationType));
                 break;
@@ -190,10 +193,12 @@ public final class PlayersTable extends JTable {
         public Component getTableCellRendererComponent(JTable table,
             Object value, boolean isSelected, boolean hasFocus,
             int row, int column) {
-            box.setSelectedItem(value);
-            final NationState nationState = (NationState)value;
-            Utility.localizeToolTip(this, StringTemplate
-                .key(nationState.getShortDescriptionKey()));
+            if (value != null) { // FIXME: Why does this happen?
+                box.setSelectedItem(value);
+                final NationState nationState = (NationState)value;
+                Utility.localizeToolTip(box, StringTemplate
+                    .key(nationState.getShortDescriptionKey()));
+            }
             return box;
         }
     }
@@ -208,20 +213,27 @@ public final class PlayersTable extends JTable {
                 });
         private final JComboBox<NationState> allStateBox
             = new JComboBox<>(NationState.values());
-        private JComboBox activeBox;
-
-        private final ActionListener listener = (ActionEvent ae) -> {
-            stopCellEditing();
-        };
+        private JComboBox<NationState> activeBox = null;
 
 
         public AvailableCellEditor() {
+            ActionListener listener = (ActionEvent ae) -> {
+                stopCellEditing();
+            };
             aiStateBox.setRenderer(new NationStateRenderer());
             aiStateBox.addActionListener(listener);
             allStateBox.setRenderer(new NationStateRenderer());
             allStateBox.addActionListener(listener);
         }
 
+        private JComboBox<NationState> getActiveBox(int row) {
+            NationType nationType = ((Nation) getValueAt(row, NATION_COLUMN))
+                .getType();
+            this.activeBox = (nationType instanceof EuropeanNationType)
+                ? this.allStateBox
+                : this.aiStateBox;
+            return this.activeBox;
+        }
 
         // Implement AbstractCellEditor
 
@@ -231,17 +243,16 @@ public final class PlayersTable extends JTable {
         @Override
         public Component getTableCellEditorComponent(JTable table,
             Object value, boolean isSelected, int row, int column) {
-            NationType nationType = ((Nation) getValueAt(row, NATION_COLUMN))
-                .getType();
-            activeBox = (nationType instanceof EuropeanNationType)
-                ? allStateBox
-                : aiStateBox;
-            return activeBox;
+            return getActiveBox(row);
         }
 
+        /**
+         * {@inheritDoc}
+         */
         @Override
         public Object getCellEditorValue() {
-            return activeBox.getSelectedItem();
+            return (this.activeBox == null) ? null
+                : this.activeBox.getSelectedItem();
         }
     }
 
@@ -319,8 +330,8 @@ public final class PlayersTable extends JTable {
             Nation nation = (Nation) value;
             setText(Messages.message(StringTemplate.template("countryName")
                     .add("%nation%", Messages.nameKey(nation.getId()))));
-            setIcon(new ImageIcon(gui.getImageLibrary()
-                    .getSmallerMiscIconImage(nation)));
+            setIcon(new ImageIcon(gui.getFixedImageLibrary()
+                    .getUnscaledSmallerNationImage(nation)));
             return this;
         }
     }
@@ -335,10 +346,8 @@ public final class PlayersTable extends JTable {
          */
         @Override
         public Component getListCellRendererComponent(JList<? extends NationState> list,
-                                                      NationState value,
-                                                      int index,
-                                                      boolean isSelected,
-                                                      boolean cellHasFocus) {
+            NationState value, int index, boolean isSelected,
+            boolean cellHasFocus) {
             setText(Messages.getName(value));
             return this;
         }
@@ -385,7 +394,7 @@ public final class PlayersTable extends JTable {
         }
     }
 
-    private final class PlayerCellEditor extends AbstractCellEditor
+    private static final class PlayerCellEditor extends AbstractCellEditor
         implements TableCellEditor {
 
         private final JButton button = Utility.localizedButton("select");
@@ -426,42 +435,40 @@ public final class PlayersTable extends JTable {
 
         private final List<Nation> nations;
 
-        private final Map<Nation, Player> players;
+        private final Map<Nation, Player> nationMap;
 
 
         /**
-         * A standard constructor.
+         * Create a new PlayersTableModel.
          *
-         * @param preGameController The <code>PreGameController</code>
+         * @param preGameController The {@code PreGameController}
          *     to use notify of updates.
-         * @param nationOptions The current <code>NationOptions</code>.
-         * @param thisPlayer The <code>Player</code> that owns the client.
+         * @param nationOptions The current {@code NationOptions}.
+         * @param thisPlayer The {@code Player} that owns the client.
          */
         public PlayersTableModel(PreGameController preGameController,
                                  NationOptions nationOptions,
                                  Player thisPlayer) {
+            final Specification spec = thisPlayer.getSpecification();
             this.preGameController = preGameController;
             this.nationOptions = nationOptions;
             this.thisPlayer = thisPlayer;
-            nations = new ArrayList<>();
-            players = new HashMap<>();
-            for (Nation nation : thisPlayer.getSpecification().getNations()) {
-                if (nation.isUnknownEnemy()) continue;
-                NationState state = nationOptions.getNations().get(nation);
-                if (state != null) {
-                    nations.add(nation);
-                    players.put(nation, null);
-                }
-            }
-            players.put(thisPlayer.getNation(), thisPlayer);
+            final Predicate<Nation> nationPred = n -> !n.isUnknownEnemy()
+                && nationOptions.getNations().get(n) != null;
+            this.nations = transform(spec.getNations(), nationPred);
+            this.nationMap = new HashMap<>(this.nations.size());
+            for (Nation n : this.nations) this.nationMap.put(n, null);
+            this.nationMap.put(thisPlayer.getNation(), thisPlayer);
         }
 
+
+        /**
+         * Update the nation map following any change.
+         */
         public void update() {
-            for (Nation nation : nations) {
-                players.put(nation, null);
-            }
-            for (Player player : thisPlayer.getGame().getLivePlayers(null)) {
-                players.put(player.getNation(), player);
+            for (Nation n : this.nations) nationMap.put(n, null);
+            for (Player p : thisPlayer.getGame().getLivePlayerList()) {
+                nationMap.put(p.getNation(), p);
             }
             fireTableDataChanged();
         }
@@ -470,7 +477,7 @@ public final class PlayersTable extends JTable {
          * Gets the class of the objects in the given column.
          *
          * @param column The column to return the class of.
-         * @return The <code>Class</code> of the objects in the given column.
+         * @return The {@code Class} of the objects in the given column.
          */
         @Override
         public Class<?> getColumnClass(int column) {
@@ -538,12 +545,12 @@ public final class PlayersTable extends JTable {
                 case AVAILABILITY_COLUMN:
                     return nationOptions.getNationState(nation);
                 case ADVANTAGE_COLUMN:
-                    return (players.get(nation) == null) ? nation.getType()
-                        : players.get(nation).getNationType();
+                    return (nationMap.get(nation) == null) ? nation.getType()
+                        : nationMap.get(nation).getNationType();
                 case COLOR_COLUMN:
                     return nation.getColor();
                 case PLAYER_COLUMN:
-                    return players.get(nation);
+                    return nationMap.get(nation);
                 }
             }
             return null;
@@ -560,7 +567,7 @@ public final class PlayersTable extends JTable {
         public boolean isCellEditable(int row, int column) {
             if (row >= 0 && row < getRowCount()) {
                 Nation nation = nations.get(row);
-                boolean ownRow = thisPlayer == players.get(nation)
+                boolean ownRow = thisPlayer == nationMap.get(nation)
                     && !thisPlayer.isReady();
                 switch (column) {
                 case AVAILABILITY_COLUMN:
@@ -577,7 +584,7 @@ public final class PlayersTable extends JTable {
                     return nation.getType() instanceof EuropeanNationType;
                 case PLAYER_COLUMN:
                     return nation.getType() instanceof EuropeanNationType
-                        && players.get(nation) == null;
+                        && nationMap.get(nation) == null;
                 }
             }
             return false;
@@ -599,6 +606,7 @@ public final class PlayersTable extends JTable {
                 switch (column) {
                 case ADVANTAGE_COLUMN:
                     preGameController.setNationType((NationType)value);
+                    update();
                     break;
                 case AVAILABILITY_COLUMN:
                     preGameController.setAvailable(nations.get(row),
@@ -607,6 +615,7 @@ public final class PlayersTable extends JTable {
                     break;
                 case COLOR_COLUMN:
                     preGameController.setColor(nation, (Color)value);
+                    update();
                     break;
                 case PLAYER_COLUMN:
                     if (nationOptions.getNationState(nation)
@@ -646,9 +655,9 @@ public final class PlayersTable extends JTable {
     /**
      * Creates a players table.
      *
-     * @param freeColClient The <code>FreeColClient</code> for the game.
-     * @param nationOptions The <code>NationOptions</code> for this game.
-     * @param myPlayer The client <code>Player</code>.
+     * @param freeColClient The {@code FreeColClient} for the game.
+     * @param nationOptions The {@code NationOptions} for this game.
+     * @param myPlayer The client {@code Player}.
      */
     public PlayersTable(final FreeColClient freeColClient,
                         NationOptions nationOptions, Player myPlayer) {
@@ -659,7 +668,8 @@ public final class PlayersTable extends JTable {
 
         setModel(new PlayersTableModel(freeColClient.getPreGameController(),
                  nationOptions, myPlayer));
-        setRowHeight(47);
+        setRowHeight((int) (FontLibrary.getFontScaling() * 47));
+        getTableHeader().setReorderingAllowed(false);
 
         JButton nationButton = Utility.localizedButton("nation");
         nationButton.addActionListener((ActionEvent ae) -> {
