@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015   The FreeCol Team
+ *  Copyright (C) 2002-2022   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -23,17 +23,26 @@ import java.awt.Color;
 import java.awt.Cursor;
 import java.awt.Dimension;
 import java.awt.Font;
+import java.awt.GridLayout;
 import java.awt.Insets;
+import java.net.Inet4Address;
+import java.net.InetAddress;
+import java.net.NetworkInterface;
+import java.net.SocketException;
+import java.util.List;
+import java.util.stream.Collectors;
 
 import javax.swing.BorderFactory;
 import javax.swing.Icon;
 import javax.swing.ImageIcon;
 import javax.swing.JButton;
 import javax.swing.JCheckBoxMenuItem;
+import javax.swing.JComboBox;
 import javax.swing.JComponent;
 import javax.swing.JLabel;
 import javax.swing.JMenu;
 import javax.swing.JMenuItem;
+import javax.swing.JPanel;
 import javax.swing.JRadioButtonMenuItem;
 import javax.swing.JTextArea;
 import javax.swing.JTextPane;
@@ -47,15 +56,18 @@ import javax.swing.text.StyleConstants;
 import javax.swing.text.StyleContext;
 
 import net.sf.freecol.client.gui.FontLibrary;
+import net.sf.freecol.client.gui.ImageLibrary;
+import net.sf.freecol.client.gui.panel.FreeColButton.ButtonStyle;
+import net.sf.freecol.client.gui.plaf.FreeColComboBoxRenderer;
 import net.sf.freecol.common.i18n.Messages;
 import net.sf.freecol.common.model.FreeColGameObject;
+import net.sf.freecol.common.model.FreeColSpecObjectType;
 import net.sf.freecol.common.model.Location;
 import net.sf.freecol.common.model.Named;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Settlement;
 import net.sf.freecol.common.model.StringTemplate;
 import net.sf.freecol.common.model.Unit;
-import net.sf.freecol.common.resources.ResourceManager;
 
 
 /**
@@ -63,71 +75,24 @@ import net.sf.freecol.common.resources.ResourceManager;
  */
 public final class Utility {
 
-
-    /** The color to use for borders. */
-    public static final Color BORDER_COLOR
-        = ResourceManager.getColor("color.border.LookAndFeel");
-
-    /** The color to use for links. */
-    public static final Color LINK_COLOR
-        = ResourceManager.getColor("color.link.LookAndFeel");
-
-    /** The color to use for things the player probably should not do. */
-    public static final Color WARNING_COLOR
-        = ResourceManager.getColor("color.warning.LookAndFeel");
-
-
     /** Useful static borders. */
-    public static final Border TRIVIAL_LINE_BORDER
-        = BorderFactory.createLineBorder(BORDER_COLOR);
-
     public static final Border BEVEL_BORDER
         = BorderFactory.createBevelBorder(BevelBorder.LOWERED);
 
-    public static final Border COLOR_CELL_BORDER = BorderFactory
-        .createCompoundBorder(
-            BorderFactory.createMatteBorder(5, 10, 5, 10,
-                new ImageIcon(ResourceManager.getImage("image.background.ColorCellRenderer"))),
-            BorderFactory.createLineBorder(BORDER_COLOR));
-
-    public static final Border DIALOG_BORDER = BorderFactory
-        .createCompoundBorder(TRIVIAL_LINE_BORDER, blankBorder(10, 20, 10, 20));
-
     public static final Border ETCHED_BORDER
         = BorderFactory.createEtchedBorder();
 
-    public static final Border PRODUCTION_BORDER = BorderFactory
-        .createCompoundBorder(BorderFactory.createMatteBorder(1, 0, 0, 0,
-                                                              BORDER_COLOR),
-                              blankBorder(2, 2, 2, 2));
-
+    // FIXME: color should be a Resource
     public static final Border PROGRESS_BORDER = BorderFactory
         .createLineBorder(new Color(122, 109, 82));
 
-    public static final Border SIMPLE_LINE_BORDER = BorderFactory
-        .createCompoundBorder(TRIVIAL_LINE_BORDER, blankBorder(5, 5, 5, 5));
-
-    // The borders to use for table cells
-    public static final Border TOPCELLBORDER
-        = BorderFactory.createCompoundBorder(
-            BorderFactory.createMatteBorder(1, 0, 1, 1, BORDER_COLOR),
-            BorderFactory.createEmptyBorder(2, 2, 2, 2));
-
-    public static final Border CELLBORDER
-        = BorderFactory.createCompoundBorder(
-            BorderFactory.createMatteBorder(0, 0, 1, 1, BORDER_COLOR),
-            BorderFactory.createEmptyBorder(2, 2, 2, 2));
-
-    public static final Border LEFTCELLBORDER
-        = BorderFactory.createCompoundBorder(
-            BorderFactory.createMatteBorder(0, 1, 1, 1, BORDER_COLOR),
-            BorderFactory.createEmptyBorder(2, 2, 2, 2));
-
-    public static final Border TOPLEFTCELLBORDER
-        = BorderFactory.createCompoundBorder(
-            BorderFactory.createMatteBorder(1, 1, 1, 1, BORDER_COLOR),
-            BorderFactory.createEmptyBorder(2, 2, 2, 2));
-
+    private static Border COLOR_CELL_BORDER = null,
+        DIALOG_BORDER = null,
+        SIMPLE_LINE_BORDER = null,
+        TOPCELLBORDER = null,
+        CELLBORDER = null,
+        LEFTCELLBORDER = null,
+        TOPLEFTCELLBORDER = null;
 
     /** How many columns (em-widths) to use in the text area. */
     private static final int DEFAULT_TEXT_COLUMNS = 20;
@@ -136,8 +101,13 @@ public final class Utility {
     public static final Insets EMPTY_MARGIN = new Insets(0, 0, 0, 0);
 
     /** A style context to use for panels and dialogs. */
-    public static StyleContext STYLE_CONTEXT = null;
+    private static StyleContext STYLE_CONTEXT = null;
 
+    /** Font specification for panel titles. */
+    public static String FONTSPEC_TITLE = "header-plain-max";
+    /** Font specification for panel subtitles. */
+    public static String FONTSPEC_SUBTITLE = "header-plain-large";
+    
     public static void initStyleContext(Font font) {
         Style defaultStyle = StyleContext.getDefaultStyleContext()
             .getStyle(StyleContext.DEFAULT_STYLE);
@@ -148,27 +118,139 @@ public final class Utility {
         StyleConstants.setFontSize(regular, font.getSize());
 
         Style buttonStyle = STYLE_CONTEXT.addStyle("button", regular);
-        StyleConstants.setForeground(buttonStyle, LINK_COLOR);
+        StyleConstants.setForeground(buttonStyle, getLinkColor());
 
         Style right = STYLE_CONTEXT.addStyle("right", regular);
         StyleConstants.setAlignment(right, StyleConstants.ALIGN_RIGHT);
     }
 
+    /** 
+     * Get the standard border color.
+     *
+     * @return The border {@code Color}.
+     */
+    public static Color getBorderColor() {
+        return ImageLibrary.getColor("color.border.LookAndFeel",
+                                     Color.BLACK);
+    }
+    
+    /** 
+     * Get the color to use for links.
+     *
+     * @return The link {@code Color}.
+     */
+    public static Color getLinkColor() {
+        return ImageLibrary.getColor("color.link.LookAndFeel", Color.BLUE);
+    }
 
+    /** 
+     * Get the color to use for things the player probably should not do.
+     *
+     * @return The warning {@code Color}.
+     */
+    public static Color getWarningColor() {
+        return ImageLibrary.getColor("color.warning.LookAndFeel", Color.RED);
+    }
+
+    // Borders that depend on Resources
+    
+    public static synchronized Border getColorCellBorder() {
+        if (COLOR_CELL_BORDER == null) {
+            ImageIcon icon = new ImageIcon(ImageLibrary
+                .getColorCellRendererBackground());
+            COLOR_CELL_BORDER = BorderFactory
+                .createCompoundBorder(BorderFactory
+                    .createMatteBorder(5, 10, 5, 10, icon),
+                    BorderFactory.createLineBorder(getBorderColor()));
+        }
+        return COLOR_CELL_BORDER;
+    }
+    
+    public static synchronized Border getDialogBorder() {
+        if (DIALOG_BORDER == null) {
+            DIALOG_BORDER = BorderFactory
+                .createCompoundBorder(getTrivialLineBorder(),
+                                      blankBorder(10, 20, 10, 20));
+        }
+        return DIALOG_BORDER;
+    }
+
+    public static synchronized Border getProductionBorder() {
+        return BorderFactory.createCompoundBorder(BorderFactory
+            .createMatteBorder(1, 0, 0, 0, getBorderColor()),
+                               blankBorder(2, 2, 2, 2));
+    }
+    
+    public static synchronized Border getSimpleLineBorder() {
+        if (SIMPLE_LINE_BORDER == null) {
+            SIMPLE_LINE_BORDER = BorderFactory
+                .createCompoundBorder(getTrivialLineBorder(),
+                                      blankBorder(5, 5, 5, 5));
+        }
+        return SIMPLE_LINE_BORDER;
+    }
+
+    public static synchronized Border getTrivialLineBorder() {
+        return BorderFactory.createLineBorder(getBorderColor());
+    }
+
+    // The borders to use for table cells
+
+    public static synchronized Border getTopCellBorder() {
+        if (TOPCELLBORDER == null) {
+            TOPCELLBORDER = BorderFactory
+                .createCompoundBorder(BorderFactory
+                    .createMatteBorder(1, 0, 1, 1, getBorderColor()),
+                    BorderFactory.createEmptyBorder(2, 2, 2, 2));
+        }
+        return TOPCELLBORDER;
+    }
+
+    public static synchronized Border getCellBorder() {
+        if (CELLBORDER == null) {
+            CELLBORDER = BorderFactory
+                .createCompoundBorder(BorderFactory
+                    .createMatteBorder(0, 0, 1, 1, getBorderColor()),
+                    BorderFactory.createEmptyBorder(2, 2, 2, 2));
+        }
+        return CELLBORDER;
+    }
+
+    public static synchronized Border getLeftCellBorder() {
+        if (LEFTCELLBORDER == null) {
+            LEFTCELLBORDER = BorderFactory
+                .createCompoundBorder(BorderFactory
+                    .createMatteBorder(0, 1, 1, 1, getBorderColor()),
+                    BorderFactory.createEmptyBorder(2, 2, 2, 2));
+        }
+        return LEFTCELLBORDER;
+    }
+
+    public static synchronized Border getTopLeftCellBorder() {
+        if (TOPLEFTCELLBORDER == null) {
+            TOPLEFTCELLBORDER = BorderFactory
+                .createCompoundBorder(BorderFactory
+                    .createMatteBorder(1, 1, 1, 1, getBorderColor()),
+                    BorderFactory.createEmptyBorder(2, 2, 2, 2));
+        }
+        return TOPLEFTCELLBORDER;
+    }
+
+    
     /**
      * Return a button suitable for linking to another panel
      * (e.g. ColopediaPanel).
      *
-     * @param text a <code>String</code> value
-     * @param icon an <code>Icon</code> value
-     * @param action a <code>String</code> value
-     * @return a <code>JButton</code> value
+     * @param text a {@code String} value
+     * @param icon an {@code Icon} value
+     * @param action a {@code String} value
+     * @return a {@code JButton} value
      */
     public static JButton getLinkButton(String text, Icon icon, String action) {
-        JButton button = new JButton(text, icon);
+        JButton button = new FreeColButton(text, icon).withButtonStyle(ButtonStyle.TRANSPARENT);
         button.setMargin(EMPTY_MARGIN);
         button.setOpaque(false);
-        button.setForeground(LINK_COLOR);
+        button.setForeground(getLinkColor());
         button.setAlignmentY(0.8f);
         button.setBorder(blankBorder(0, 0, 0, 0));
         button.setActionCommand(action);
@@ -185,27 +267,33 @@ public final class Utility {
      *
      * @param key The message key to make a link for.
      * @param val The text for the link.
-     * @param player The <code>Player</code> to make a link for.
-     * @param source The message source <code>FreeColGameObject</code>.
-     * @return A <code>JButton</code> for the link, or null if no good
+     * @param player The {@code Player} to make a link for.
+     * @param source The message source {@code FreeColGameObject}.
+     * @return A {@code JButton} for the link, or null if no good
      *     choice found.
      */
     public static JButton getMessageButton(String key, String val,
         Player player, FreeColGameObject source) {
         FreeColGameObject link = null;
-        if ("%colony%".equals(key) || key.endsWith("Colony%")) {
+        if ("%colony%".equals(key) || key.endsWith("Colony%")
+            || "%settlement%".equals(key)) {
             Settlement settlement = player.getGame().getSettlementByName(val);
             link = (settlement == null) ? null
                 : (player.owns(settlement)) ? settlement
                 : settlement.getTile();
         } else if ("%europe%".equals(key) || "%market%".equals(key)) {
             link = player.getEurope();
-        } else if ("%location%".equals(key) || key.endsWith("Location%")) {
+        } else if ("%location%".equals(key)
+            || "%repairLocation%".equals(key)) {
             if (source instanceof Location) {
                 link = source.getLinkTarget(player);
             }
-        } else if ("%unit%".equals(key) || key.endsWith("Unit%")) {
-            if (source instanceof Unit) {
+        } else if ("%unit%".equals(key)) {
+            if (source instanceof Unit && player.owns((Unit)source)) {
+                link = source.getLinkTarget(player);
+            }
+        } else if ("%enemyUnit%".equals(key)) {
+            if (source instanceof Unit && !player.owns((Unit)source)) {
                 link = source.getLinkTarget(player);
             }
         }
@@ -220,7 +308,7 @@ public final class Utility {
      * @param text The text to display in the text area.
      * @return A suitable text area.
      */
-    public static JTextArea createTextArea(String text) {
+    private static JTextArea createTextArea(String text) {
         JTextArea textArea = new JTextArea(text);
         textArea.setOpaque(false);
         textArea.setLineWrap(true);
@@ -270,9 +358,9 @@ public final class Utility {
     }
 
     /**
-     * Get a <code>JTextPane</code> with default styles.
+     * Get a {@code JTextPane} with default styles.
      *
-     * @return The default <code>JTextPane</code> to use.
+     * @return The default {@code JTextPane} to use.
      */
     public static JTextPane getDefaultTextPane() {
         DefaultStyledDocument document
@@ -286,10 +374,10 @@ public final class Utility {
     }
 
     /**
-     * Get a <code>JTextPane</code> with default styles and given text.
+     * Get a {@code JTextPane} with default styles and given text.
      *
      * @param text The text to display.
-     * @return A suitable <code>JTextPane</code>.
+     * @return A suitable {@code JTextPane}.
      */
     public static JTextPane getDefaultTextPane(String text) {
         JTextPane textPane = getDefaultTextPane();
@@ -316,12 +404,37 @@ public final class Utility {
                 blankBorder(top, left, bottom, right),
                 component.getBorder()));
     }
+    
+    /**
+     * Returns a combo box for selecting a possible server address.
+     * @return The combo box.
+     */
+    public static JComboBox<InetAddress> createServerInetAddressBox() {
+        final List<InetAddress> serverAddresses = getPossibleServerAddresses();
+        final JComboBox<InetAddress> serverAddressBox = new JComboBox<>(serverAddresses.toArray(new InetAddress[0]));
+        serverAddressBox.setRenderer(new FreeColComboBoxRenderer<>());
+        return serverAddressBox;
+    }
+
+    private static List<InetAddress> getPossibleServerAddresses() {
+        List<InetAddress> serverAddresses;
+        try {
+            serverAddresses = NetworkInterface.networkInterfaces()
+                    .flatMap(NetworkInterface::inetAddresses)
+                    .filter(ia -> ia instanceof Inet4Address)
+                    .collect(Collectors.toList());
+        } catch (SocketException e) {
+            serverAddresses = List.of(Inet4Address.getLoopbackAddress());
+        }
+        return serverAddresses;
+    }
+
 
     /**
      * Localize the a titled border.
      *
-     * @param component The <code>JComponent</code> to localize.
-     * @param template The <code>StringTemplate</code> to use.
+     * @param component The {@code JComponent} to localize.
+     * @param template The {@code StringTemplate} to use.
      */
     public static void localizeBorder(JComponent component,
                                       StringTemplate template) {
@@ -332,18 +445,30 @@ public final class Utility {
     /**
      * Get a titled border for a Named object.
      *
-     * @param named The <code>Named</code> to use.
-     * @return The <code>TitledBorder</code>.
+     * @param named The {@code Named} to use.
+     * @return The {@code TitledBorder}.
      */
     public static TitledBorder localizedBorder(Named named) {
         return localizedBorder(named.getNameKey());
     }
 
+    /**
+     * Get a titled border for a Named object and a given
+     * colored line border.
+     *
+     * @param named The {@code Named} to use.
+     * @param color The color to use.
+     * @return The {@code TitledBorder}.
+     */
+    public static TitledBorder localizedBorder(Named named, Color color) {
+        return localizedBorder(named.getNameKey(), color);
+    }
+
     /**
      * Get a titled border with Messages.message(key) as text.
      *
      * @param key The key to use.
-     * @return The <code>TitledBorder</code>.
+     * @return The {@code TitledBorder}.
      */
     public static TitledBorder localizedBorder(String key) {
         return BorderFactory.createTitledBorder(BorderFactory
@@ -356,7 +481,7 @@ public final class Utility {
      *
      * @param key The key to use.
      * @param color The color to use.
-     * @return The <code>TitledBorder</code>.
+     * @return The {@code TitledBorder}.
      */
     public static TitledBorder localizedBorder(String key, Color color) {
         return BorderFactory.createTitledBorder(BorderFactory
@@ -367,17 +492,17 @@ public final class Utility {
      * Get a JButton with Messages.message(key) as text.
      *
      * @param key The key to use.
-     * @return The <code>JButton</code>.
+     * @return The {@code JButton}.
      */
-    public static JButton localizedButton(String key) {
-        return new JButton(Messages.message(key));
+    public static FreeColButton localizedButton(String key) {
+        return new FreeColButton(Messages.message(key));
     }
 
     /**
      * Get a JButton with Messages.message(template) as text.
      *
-     * @param template The <code>StringTemplate</code> to use.
-     * @return The <code>JButton</code>.
+     * @param template The {@code StringTemplate} to use.
+     * @return The {@code JButton}.
      */
     public static JButton localizedButton(StringTemplate template) {
         return new JButton(Messages.message(template));
@@ -388,7 +513,7 @@ public final class Utility {
      *
      * @param key The key to use.
      * @param value The initial value.
-     * @return The <code>JCheckBoxMenuItem</code>.
+     * @return The {@code JCheckBoxMenuItem}.
      */
     public static JCheckBoxMenuItem localizedCheckBoxMenuItem(String key,
                                                               boolean value) {
@@ -399,12 +524,12 @@ public final class Utility {
      * Gets a default header for panels containing a localized message.
      *
      * @param key The message key to use.
-     * @param small If true, use a smaller font.
-     * @return A suitable <code>JLabel</code>.
+     * @param fontSpec A font-specification for the font to use.
+     * @return A suitable {@code JLabel}.
      */
-    public static JLabel localizedHeader(String key, boolean small) {
+    public static JLabel localizedHeader(String key, String fontSpec) {
         JLabel header = localizedHeaderLabel(key, SwingConstants.CENTER,
-            (small ? FontLibrary.FontSize.SMALL : FontLibrary.FontSize.BIG));
+                                             fontSpec);
         header.setBorder(BorderFactory.createEmptyBorder(20, 0, 20, 0));
         return header;
     }
@@ -414,51 +539,49 @@ public final class Utility {
      *
      * @param key The message key to use.
      * @param alignment The alignment.
-     * @param size The font size.
-     * @return A suitable <code>JLabel</code>.
+     * @param fontSpec A font specification.
+     * @return A suitable {@code JLabel}.
      */
     public static JLabel localizedHeaderLabel(String key, int alignment,
-                                              FontLibrary.FontSize size) {
+                                              String fontSpec) {
         String text = Messages.message(key);
         JLabel header = new JLabel(text, alignment);
-        header.setFont(FontLibrary.createCompatibleFont(
-            text, FontLibrary.FontType.HEADER, size));
+        header.setFont(FontLibrary.getScaledFont(fontSpec, text));
         header.setOpaque(false);
         return header;
     }
 
     public static JLabel localizedHeaderLabel(StringTemplate template,
                                               int alignment,
-                                              FontLibrary.FontSize size) {
+                                              String fontSpec) {
         String text = Messages.message(template);
         JLabel header = new JLabel(text, alignment);
-        header.setFont(FontLibrary.createCompatibleFont(
-            text, FontLibrary.FontType.HEADER, size));
+        header.setFont(FontLibrary.getScaledFont(fontSpec, text));
         header.setOpaque(false);
         return header;
     }
 
     public static JLabel localizedHeaderLabel(Named named,
-                                              FontLibrary.FontSize size) {
+                                              String fontSpec) {
         return localizedHeaderLabel(named.getNameKey(),
-                                    SwingConstants.LEADING, size);
+                                    SwingConstants.LEADING, fontSpec);
     }
 
     /**
-     * Get a JLabel with a named object.
+     * Get a JLabel for a FreeColSpecObjectType.
      *
-     * @param named The <code>Named</code> to use.
-     * @return The <code>JLabel</code>.
+     * @param fcgot The {@code FreeColSpecObjectType} to use.
+     * @return The {@code JLabel}.
      */
-    public static JLabel localizedLabel(Named named) {
-        return localizedLabel(named.getNameKey());
+    public static JLabel localizedLabel(FreeColSpecObjectType fcgot) {
+        return localizedLabel(fcgot.getNameKey());
     }
 
     /**
      * Get a JLabel with Messages.message(key) as text.
      *
      * @param key The key to use.
-     * @return The <code>JLabel</code>.
+     * @return The {@code JLabel}.
      */
     public static JLabel localizedLabel(String key) {
         return localizedLabel(StringTemplate.key(key));
@@ -467,8 +590,8 @@ public final class Utility {
     /**
      * Get a JLabel with Messages.message(template) as text.
      *
-     * @param template The <code>StringTemplate</code> to use.
-     * @return The <code>JLabel</code>.
+     * @param template The {@code StringTemplate} to use.
+     * @return The {@code JLabel}.
      */
     public static JLabel localizedLabel(StringTemplate template) {
         JLabel label = new JLabel(Messages.message(template));
@@ -479,10 +602,10 @@ public final class Utility {
     /**
      * Get a JLabel with Messages.message(template) as text.
      *
-     * @param template The <code>StringTemplate</code> to use.
+     * @param template The {@code StringTemplate} to use.
      * @param icon The icon to use.
      * @param alignment The alignment.
-     * @return The <code>JLabel</code>.
+     * @return The {@code JLabel}.
      */
     public static JLabel localizedLabel(StringTemplate template, Icon icon,
                                         int alignment) {
@@ -495,7 +618,7 @@ public final class Utility {
      * Get a JMenu with Messages.message(key) as text.
      *
      * @param key The key to use.
-     * @return The <code>JMenu</code>.
+     * @return The {@code JMenu}.
      */
     public static JMenu localizedMenu(String key) {
         return new JMenu(Messages.message(key));
@@ -504,8 +627,8 @@ public final class Utility {
     /**
      * Get a JMenu with Messages.message(template) as text.
      *
-     * @param template The <code>StringTemplate</code> to use.
-     * @return The <code>JMenu</code>.
+     * @param template The {@code StringTemplate} to use.
+     * @return The {@code JMenu}.
      */
     public static JMenu localizedMenu(StringTemplate template) {
         return new JMenu(Messages.message(template));
@@ -515,7 +638,7 @@ public final class Utility {
      * Get a JMenuItem with Messages.message(key) as text.
      *
      * @param key The key to use.
-     * @return The <code>JMenuItem</code>.
+     * @return The {@code JMenuItem}.
      */
     public static JMenuItem localizedMenuItem(String key) {
         return localizedMenuItem(key, null);
@@ -526,7 +649,7 @@ public final class Utility {
      *
      * @param key The key to use.
      * @param icon The icon to use.
-     * @return The <code>JMenuItem</code>.
+     * @return The {@code JMenuItem}.
      */
     public static JMenuItem localizedMenuItem(String key, Icon icon) {
         return new JMenuItem(Messages.message(key), icon);
@@ -535,8 +658,8 @@ public final class Utility {
     /**
      * Get a JMenuItem with Messages.message(template) as text.
      *
-     * @param template The <code>StringTemplate</code> to use.
-     * @return The <code>JMenuItem</code>.
+     * @param template The {@code StringTemplate} to use.
+     * @return The {@code JMenuItem}.
      */
     public static JMenuItem localizedMenuItem(StringTemplate template) {
         return localizedMenuItem(template, null);
@@ -545,9 +668,9 @@ public final class Utility {
     /**
      * Get a JMenuItem with Messages.message(template) as text.
      *
-     * @param template The <code>StringTemplate</code> to use.
+     * @param template The {@code StringTemplate} to use.
      * @param icon The icon to use.
-     * @return The <code>JMenuItem</code>.
+     * @return The {@code JMenuItem}.
      */
     public static JMenuItem localizedMenuItem(StringTemplate template,
                                               Icon icon) {
@@ -557,9 +680,9 @@ public final class Utility {
     /**
      * Get a JRadioButtonMenuItem with Messages.message(template) as text.
      *
-     * @param template The <code>StringTemplate</code> to generate the text.
+     * @param template The {@code StringTemplate} to generate the text.
      * @param value The initial value.
-     * @return The <code>JRadioButtonMenuItem</code>.
+     * @return The {@code JRadioButtonMenuItem}.
      */
     public static JRadioButtonMenuItem localizedRadioButtonMenuItem(StringTemplate template,
                                                                     boolean value) {
@@ -570,7 +693,7 @@ public final class Utility {
      * Get a text area containing a localized message.
      *
      * @param key The message key.
-     * @return A suitable <code>JTextArea</code>.
+     * @return A suitable {@code JTextArea}.
      */
     public static JTextArea localizedTextArea(String key) {
         return localizedTextArea(StringTemplate.key(key));
@@ -581,7 +704,7 @@ public final class Utility {
      *
      * @param key The message key.
      * @param columns The em-width number of columns to display the text in.
-     * @return A suitable <code>JTextArea</code>.
+     * @return A suitable {@code JTextArea}.
      */
     public static JTextArea localizedTextArea(String key, int columns) {
         return localizedTextArea(StringTemplate.key(key), columns);
@@ -590,8 +713,8 @@ public final class Utility {
     /**
      * Get a text area containing a localized message.
      *
-     * @param template The <code>StringTemplate</code> to use.
-     * @return A suitable <code>JTextArea</code>.
+     * @param template The {@code StringTemplate} to use.
+     * @return A suitable {@code JTextArea}.
      */
     public static JTextArea localizedTextArea(StringTemplate template) {
         return localizedTextArea(template, DEFAULT_TEXT_COLUMNS);
@@ -600,38 +723,50 @@ public final class Utility {
     /**
      * Get a text area containing a localized message.
      *
-     * @param template The <code>StringTemplate</code> to use.
+     * @param template The {@code StringTemplate} to use.
      * @param columns The em-width number of columns to display the text in.
-     * @return A suitable <code>JTextArea</code>.
+     * @return A suitable {@code JTextArea}.
      */
     public static JTextArea localizedTextArea(StringTemplate template,
                                               int columns) {
         return getDefaultTextArea(Messages.message(template), columns);
     }
 
+    /**
+     * Get a panel with a localized message and icon.
+     *
+     * @param template The {@code StringTemplate} to use.
+     * @param icon An {@code ImageIcon} to use.
+     * @return The resulting {@code JPanel}.
+     */
+    public static JPanel localizedTextPanel(StringTemplate template,
+                                            ImageIcon icon) {
+        JPanel panel = new JPanel(new GridLayout(0, 2));
+        panel.add(new JLabel(icon));
+        panel.add(Utility.localizedTextArea(template));
+        panel.setBorder(getDialogBorder());
+        return panel;
+    }
+        
     /**
      * Localize the tool tip message for a JComponent.
      *
-     * @param comp The <code>JComponent</code> to localize.
+     * @param comp The {@code JComponent} to localize.
      * @param key The key to use.
-     * @return The original <code>JComponent</code>.
      */
-    public static JComponent localizeToolTip(JComponent comp, String key) {
+    public static void localizeToolTip(JComponent comp, String key) {
         comp.setToolTipText(Messages.message(key));
-        return comp;
     }
 
     /**
      * Localize the tool tip message for a JComponent.
      *
-     * @param comp The <code>JComponent</code> to localize.
-     * @param template The <code>StringTemplate</code> to use.
-     * @return The original <code>JComponent</code>.
+     * @param comp The {@code JComponent} to localize.
+     * @param template The {@code StringTemplate} to use.
      */
-    public static JComponent localizeToolTip(JComponent comp,
-                                             StringTemplate template) {
+    public static void localizeToolTip(JComponent comp,
+                                       StringTemplate template) {
         comp.setToolTipText(Messages.message(template));
-        return comp;
     }
 
 }
