@@ -785,7 +785,7 @@ public class DebugUtils {
         lb.add("\n->", label, "\n");
         for (Unit u : units) {
             lb.add("\n", u.getDescription(Unit.UnitLabelType.NATIONAL));
-            if (u.isDamaged()) {
+            if (u.isDamagedAndUnderForcedRepair()) {
                 lb.add(" (", Messages.message(u.getRepairLabel()), ")");
             } else {
                 lb.add("    ");
@@ -1090,6 +1090,12 @@ public class DebugUtils {
      * @param freeColClient The {@code FreeColClient} for the game.
      */
     public static void skipTurns(FreeColClient freeColClient) {
+        final FreeColServer server = freeColClient.getFreeColServer();
+        if (server != null && server.getInGameController().getSkippedTurns() > 0) {
+            freeColClient.skipTurns(0); // Clear existing skipping
+            return;
+        }
+
         freeColClient.skipTurns(0); // Clear existing skipping
 
         String response = freeColClient.getGUI().getInput(null,
