@@ -38,6 +38,7 @@ import static net.sf.freecol.common.util.CollectionUtils.*;
 import net.sf.freecol.common.util.LogBuilder;
 import static net.sf.freecol.common.util.StringUtils.*;
 import net.sf.freecol.server.FreeColServer;
+import net.sf.freecol.server.ai.military.DefensiveMap;
 
 
 /**
@@ -80,6 +81,11 @@ public class FreeColDebugger {
      */
     private static boolean normalGameFogOfWar = false;
 
+    /**
+     * Displays elements for debugging the rendering.
+     */
+    private static boolean debugRendering = false;
+    
     /** Display map coordinates? */
     private static boolean displayCoordinates = false;
 
@@ -95,6 +101,9 @@ public class FreeColDebugger {
     /** Stream for debugLog. */
     private static final AtomicReference<PrintStream> debugStream
         = new AtomicReference<PrintStream>(null);
+    
+    /** Displays the defence map for this player */
+    private static Player defenceMapPlayer = null;
 
 
     /**
@@ -290,6 +299,22 @@ public class FreeColDebugger {
     public static void signalEndDebugRun() {
         if (debugRunTurns > 0) setDebugRunTurns(0);
     }
+    
+    /**
+     * Should elements for debugging the rendering be displayed?
+     */
+    public static boolean debugRendering() {
+        return debugRendering;
+    }
+    
+    /**
+     * Displays elements for debugging the rendering.
+     *
+     * @param _debugRendering If true, displays elements for debugging the rendering.
+     */
+    public static void setDebugRendering(boolean _debugRendering) {
+        debugRendering = _debugRendering;
+    }
 
     /**
      * Should the map viewer display tile coordinates?
@@ -363,6 +388,17 @@ public class FreeColDebugger {
     public static void setDebugShowMissionInfo(boolean display) {
         showMissionInfo = display;
     }
+    
+    /**
+     * Displays the {@link DefensiveMap} for a given player.
+     */
+    public static void setDebugShowDefenceMapForPlayer(Player defenceMapPlayer) {
+        FreeColDebugger.defenceMapPlayer = defenceMapPlayer;        
+    }
+    
+    public static Player debugShowDefenceMapForPlayer() {
+        return defenceMapPlayer;
+    }
 
     /**
      * Handler for log records that include a crash.
@@ -371,7 +407,6 @@ public class FreeColDebugger {
         if (debugRunSave != null) signalEndDebugRun();
     }
 
-
     /**
      * Emergency run time log to use when the normal logging is failing.
      * It might as well be here.
