@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015   The FreeCol Team
+ *  Copyright (C) 2002-2022   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -76,6 +76,10 @@ public class NameCache {
     private static List<String> riverNames = null;
     private static final Object riverNameLock = new Object();
 
+    /** Extra rumour names. */
+    private static final Object rumourNothingLock = new Object();
+    private static List<String> rumourNothingKeys = null;
+    
     /** Season names. */
     private static List<String> seasonNames = null;
     private static final Object seasonNamesLock = new Object();
@@ -113,6 +117,20 @@ public class NameCache {
         }
     }
 
+    /**
+     * Collects all the keys with a given prefix.
+     *
+     * @param prefix The prefix to check.
+     * @param keys A list to fill with the keys found.
+     * @param suffix A suffix to add.
+     */
+    private static void collectKeys(String prefix, List<String> keys,
+                                    String suffix) {
+        String key;
+        for (int i = 0; Messages.containsKey(key = prefix
+                + Integer.toString(i) + suffix); i++) keys.add(key);
+    }
+
     /**
      * Initialize the cities of Cibola collection.
      *
@@ -151,8 +169,8 @@ public class NameCache {
     /**
      * Make a key for a player and region type.
      *
-     * @param player The <code>Player</code> to install region names for.
-     * @param type The <code>RegionType</code> to get names of.
+     * @param player The {@code Player} to install region names for.
+     * @param type The {@code RegionType} to get names of.
      * @return A key for the player and region type.
      */
     private static String makeRegionKey(Player player, RegionType type) {
@@ -163,8 +181,8 @@ public class NameCache {
     /**
      * Initialize the region names for a player.
      *
-     * @param player The <code>Player</code> to install region names for.
-     * @param type The <code>RegionType</code> to get names of.
+     * @param player The {@code Player} to install region names for.
+     * @param type The {@code RegionType} to get names of.
      */
     private static void requireRegionNames(Player player, RegionType type) {
         synchronized (regionNameLock) {
@@ -180,6 +198,19 @@ public class NameCache {
         }
     }
 
+    /**
+     * Initialize the rumourNothing collection.
+     */
+    private static void requireRumourNothingKeys() {
+        synchronized (rumourNothingLock) {
+            if (rumourNothingKeys == null) {
+                rumourNothingKeys = new ArrayList<>();
+                collectKeys("model.lostCityRumour.nothing.",
+                            rumourNothingKeys, Messages.DESCRIPTION_SUFFIX);
+            }
+        }
+    }
+
     /**
      * Initialize the riverNames collection.
      */
@@ -234,35 +265,41 @@ public class NameCache {
     /**
      * Initialize the settlement names for a player.
      *
-     * @param player The <code>Player</code> to install names for.
+     * @param player The {@code Player} to install names for.
      * @param random A pseudo-random number source.
      */
     private static void requireSettlementNames(Player player, Random random) {
         synchronized (settlementNameLock) {
             if (settlementNames.get(player) == null) {
                 List<String> names = new ArrayList<>();
-                // @compat 0.10.x
-                // Try the base names
-                collectNames(player.getNationId() + ".settlementName.", names);
-                // end @compat 0.10.x
+                if (player.isEuropean()) {
+                    String specId = player.getSpecification().getId();
+                    // The older European nation settlement names are
+                    // spec-qualified and in chronological order
+                    String cacheId = player.getNationId() + ".settlementName."
+                        + specId + ".";
+                    collectNames(cacheId, names);
 
-                // Try the spec-qualified version.
-                if (names.isEmpty()) {
-                    collectNames(player.getNationId() + ".settlementName."
-                        + player.getSpecification().getId() + ".", names);
-                }
-
-                // If still empty use the "freecol" names.
-                if (names.isEmpty()) {
-                    collectNames(player.getNationId() + ".settlementName.freecol.",
+                    // Newer European settlement names are not spec-qualified
+                    cacheId = player.getNationId() + ".settlementName.";
+                    collectNames(cacheId, names);
+                    
+                    // If still none found fall back to the "freecol" names
+                    if (names.isEmpty() && !"freecol".equals(specId)) {
+                        cacheId = player.getNationId() + ".settlementName.freecol.";
+                        collectNames(cacheId, names);
+                    }
+                } else {
+                    // Native settlement names are not spec qualified,
+                    // but have capitals and need randomization
+                    collectNames(player.getNationId() + ".settlementName.",
                                  names);
-                }
-
-                // Indians have capitals and need randomization
-                if (player.isIndian()) {
-                    capitalNames.put(player, names.remove(0));
-                    if (random != null) {
-                        randomShuffle(logger, "Settlement names", names, random);
+                    if (!names.isEmpty()) {
+                        capitalNames.put(player, names.remove(0));
+                        if (random != null) {
+                            randomShuffle(logger, "Native settlement names",
+                                          names, random);
+                        }
                     }
                 }
                 settlementNames.put(player, names);
@@ -273,9 +310,9 @@ public class NameCache {
     }
 
     /**
-     * Initialize the shipNames for a player.
+     * Initialize the ship names for a player.
      *
-     * @param player The <code>Player</code> to install names for.
+     * @param player The {@code Player} to install names for.
      * @param random A pseudo-random number source.
      */
     private static void requireShipNames(Player player, Random random) {
@@ -371,7 +408,7 @@ public class NameCache {
     /**
      * Get the new land name for a player.
      *
-     * @param player The <code>Player</code> to query.
+     * @param player The {@code Player} to query.
      * @return The new land name of a player.
      */
     public static String getNewLandName(Player player) {
@@ -384,8 +421,8 @@ public class NameCache {
      * Creates a unique region name for a player by fetching a new
      * name from the list of default names if possible.
      *
-     * @param player The <code>Player</code> to find a region name for.
-     * @param region The <code>Region</code> to name.
+     * @param player The {@code Player} to find a region name for.
+     * @param region The {@code Region} to name.
      * @return A suitable name.
      */
     public static String getRegionName(Player player, Region region) {
@@ -437,10 +474,24 @@ public class NameCache {
         return name;
     }
 
+    /**
+     * Get a random "nothing" rumour key.
+     *
+     * @param random A pseudo-random number source.
+     * @return A suitable message key.
+     */
+    public static String getRumourNothingKey(Random random) {
+        requireRumourNothingKeys();
+        synchronized (rumourNothingLock) {
+            return getRandomMember(logger, "nothingKey", rumourNothingKeys,
+                                   random);
+        }
+    }
+
     /**
      * Get a fallback settlement name for a player.
      *
-     * @param player The <code>Player</code> to get a fallback
+     * @param player The {@code Player} to get a fallback
      *     settlement name for.
      * @return A unique fallback settlement name for the player.
      */
@@ -453,7 +504,7 @@ public class NameCache {
     /**
      * Get the name of this players capital.  Only meaningful to natives.
      *
-     * @param player The <code>Player</code> to get a capital name for.
+     * @param player The {@code Player} to get a capital name for.
      * @param random An optional pseudo-random number source.
      * @return The name of this players capital.
      */
@@ -467,7 +518,7 @@ public class NameCache {
     /**
      * Get a settlement name suitable for a player.
      *
-     * @param player The <code>Player</code> to get a settlement name for.
+     * @param player The {@code Player} to get a settlement name for.
      * @param random An optional pseudo-random number source.
      * @return A new settlement name.
      */
@@ -484,7 +535,7 @@ public class NameCache {
 
         // Use the fallback name
         final String base = getFallbackSettlementName(player);
-        int i = player.getSettlements().size() + 1;
+        int i = player.getSettlementCount() + 1;
         String name = null;
         while (game.getSettlementByName(name = base + i++) != null);
         return name;
@@ -493,7 +544,7 @@ public class NameCache {
     /**
      * Puts a suggested settlement name back into the pool.
      *
-     * @param player The <code>Player</code> returning the settlement name.
+     * @param player The {@code Player} returning the settlement name.
      * @param name A formerly suggested settlement name.
      */
     public static void putSettlementName(Player player, String name) {
@@ -509,15 +560,15 @@ public class NameCache {
     /**
      * Get a new default trade route name for a player.
      *
-     * @param player The <code>Player</code> to get the name for.
+     * @param player The {@code Player} to get the name for.
      * @return A new trade route name.
      */
     public static String getTradeRouteName(Player player) {
         String base = Messages.message("nameCache.base.tradeRoute");
-        if (player.getTradeRouteByName(base) == null) return base;
+        if (player.getTradeRouteByName(base, null) == null) return base;
         String name;
         int i = 1;
-        while (player.getTradeRouteByName(name = base + i++) != null);
+        while (player.getTradeRouteByName(name = base + i++, null) != null);
         return name;
     }
 
@@ -525,16 +576,17 @@ public class NameCache {
      * Gets a new name for a unit.
      *
      * Currently only names naval units, not specific to type.
+     * REF ships should not be named.
      * FIXME: specific names for types.
      *
-     * @param player The <code>Player</code> who will own the unit.
-     * @param type The <code>UnitType</code> to choose a name for.
+     * @param player The {@code Player} who will own the unit.
+     * @param type The {@code UnitType} to choose a name for.
      * @param random A pseudo-random number source.
      * @return A name for the unit, or null if not available.
      */
     public static String getUnitName(Player player, UnitType type,
                                      Random random) {
-        if (!type.isNaval()) return null;
+        if (!type.isNaval() || player.isREF()) return null;
         String name;
 
         // Find a new name in the installed ship names if possible.
