@@ -483,6 +483,14 @@ public class FreeColDirectories {
      * @return The new autosave directory, or null if not possible.
      */
     private static File deriveAutosaveDirectory() {
+        if (saveDirectory != null
+            && saveDirectory.toPath().endsWith(AUTOSAVE_DIRECTORY)) {
+            // Do not create autosave directories inside autosave
+            // directories (BR#3276).  However, having save==autosave
+            // is not a good idea either.  Lets see who that next
+            // complaint is.
+            return saveDirectory;
+        }
         return deriveDirectory(saveDirectory, AUTOSAVE_DIRECTORY);
     }
 
@@ -579,7 +587,6 @@ public class FreeColDirectories {
         }
         // TODO: Drop trace when BR#3097b is settled
         File dir = deriveAutosaveDirectory();
-        System.err.println("Autosave directory initialized to " + dir);
         setAutosaveDirectory(dir);
         userModsDirectory = new File(getUserDataDirectory(), MODS_DIRECTORY);
         if (!insistDirectory(userModsDirectory)) userModsDirectory = null;
@@ -617,9 +624,6 @@ public class FreeColDirectories {
         // Re-establish the autosave directory if it has gone away
         if (!isGoodDirectory(autosaveDirectory)) {
             File dir = deriveAutosaveDirectory();
-            // TODO: Drop trace when BR#3097b is settled
-            System.err.println("Autosave directory " + autosaveDirectory
-                + " is broken, replacing with " + dir);
             setAutosaveDirectory(dir);
         }
         return autosaveDirectory;
@@ -1151,8 +1155,6 @@ public class FreeColDirectories {
         saveDirectory = parent;
         // TODO: Drop trace when BR#3097b is settled
         File dir = deriveAutosaveDirectory();
-        System.err.println("Autosave directory " + autosaveDirectory
-                + " follows saveDirectory change to " + dir);
         setAutosaveDirectory(dir);
         return true;
     }
