@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015   The FreeCol Team
+ *  Copyright (C) 2002-2022   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -19,7 +19,9 @@
 
 package net.sf.freecol.common.model;
 
+import java.util.AbstractMap.SimpleEntry;
 import java.util.ArrayList;
+import java.util.Comparator;
 import java.util.List;
 
 import javax.xml.stream.XMLStreamException;
@@ -31,36 +33,43 @@ import net.sf.freecol.common.io.FreeColXMLWriter;
 import net.sf.freecol.common.util.Utils;
 import static net.sf.freecol.common.util.StringUtils.*;
 
-import org.w3c.dom.Element;
-
 
 /**
  * Contains a message about a change in the model.
  */
 public class ModelMessage extends StringTemplate {
 
-    /** Constants describing the type of message. */
-    public static enum MessageType implements Named {
+    public static final String TAG = "modelMessage";
 
+    /**
+     * Constants categorizing the type of message.
+     *
+     * When this changes, be sure to update client-options.xml and the
+     * messages file appropriately.
+     */
+    public static enum MessageType implements Named {
         DEFAULT(""),
-        WARNING("model.option.guiShowWarning"),
-        SONS_OF_LIBERTY("model.option.guiShowSonsOfLiberty"),
-        GOVERNMENT_EFFICIENCY("model.option.guiShowGovernmentEfficiency"),
-        WAREHOUSE_CAPACITY("model.option.guiShowWarehouseCapacity"),
-        UNIT_IMPROVED("model.option.guiShowUnitImproved"),
-        UNIT_DEMOTED("model.option.guiShowUnitDemoted"),
-        UNIT_LOST("model.option.guiShowUnitLost"),
-        UNIT_ADDED("model.option.guiShowUnitAdded"),
         BUILDING_COMPLETED("model.option.guiShowBuildingCompleted"),
-        FOREIGN_DIPLOMACY("model.option.guiShowForeignDiplomacy"),
-        MARKET_PRICES("model.option.guiShowMarketPrices"),
-        LOST_CITY_RUMOUR(null), // Displayed during the turn
-        MISSING_GOODS("model.option.guiShowMissingGoods"),
-        TUTORIAL("model.option.guiShowTutorial"),
         COMBAT_RESULT(null), // No option, always display
-        GIFT_GOODS("model.option.guiShowGifts"),
         DEMANDS("model.option.guiShowDemands"),
-        GOODS_MOVEMENT("model.option.guiShowGoodsMovement");
+        DISASTERS("model.option.guiShowDisasters"),
+        FOREIGN_DIPLOMACY("model.option.guiShowForeignDiplomacy"),
+        GIFT_GOODS("model.option.guiShowGifts"),
+        GOODS_MOVEMENT("model.option.guiShowGoodsMovement"),
+        GOVERNMENT_EFFICIENCY("model.option.guiShowGovernmentEfficiency"),
+        LOST_CITY_RUMOUR(null), // Displayed during the turn
+        MARKET_PRICES("model.option.guiShowMarketPrices"),
+        MISSING_GOODS("model.option.guiShowMissingGoods"),
+        SONS_OF_LIBERTY("model.option.guiShowSonsOfLiberty"),
+        TUTORIAL("model.option.guiShowTutorial"),
+        UNIT_ADDED("model.option.guiShowUnitAdded"),
+        UNIT_ARRIVED("model.option.guiShowUnitArrived"),
+        UNIT_DEMOTED("model.option.guiShowUnitDemoted"),
+        UNIT_IMPROVED("model.option.guiShowUnitImproved"),
+        UNIT_LOST("model.option.guiShowUnitLost"),
+        UNIT_REPAIRED("model.option.guiShowUnitRepaired"),
+        WAREHOUSE_CAPACITY("model.option.guiShowWarehouseCapacity"),
+        WARNING("model.option.guiShowWarning");
 
         private final String optionName;
 
@@ -91,39 +100,57 @@ public class ModelMessage extends StringTemplate {
         }
     }
 
+    /** Compare messages by type. */
+    public static final Comparator<ModelMessage> messageTypeComparator
+        = (m1, m2) -> m1.getMessageType().ordinal()
+                    - m2.getMessageType().ordinal();
+
     private String sourceId;
     private String displayId;
     private MessageType messageType;
-    private boolean beenDisplayed = false;
+    private boolean displayed = false;
 
 
     /**
-     * Creates a new <code>ModelMessage</code>.
+     * Trivial constructor to allow creation with Game.newInstance.
+     */
+    public ModelMessage() {}
+        
+    /**
+     * Creates a new {@code ModelMessage}.
      *
+     * @param messageType The type of this model message.
      * @param id The object identifier.
+     * @param defaultId The default identifier.
      * @param source The source of the message. This is what the
      *               message should be associated with.
+     * @param display The {@code FreeColObject} to display.
      */
-    public ModelMessage(String id, FreeColGameObject source) {
-        this(MessageType.DEFAULT, id, source,
-             getDefaultDisplay(MessageType.DEFAULT, source));
+    public ModelMessage(MessageType messageType, String id, String defaultId,
+                        FreeColGameObject source, FreeColObject display) {
+        super(id, defaultId, TemplateType.TEMPLATE);
+
+        this.messageType = messageType;
+        this.sourceId = source.getId();
+        this.displayId = (display != null) ? display.getId() : source.getId();
     }
 
     /**
-     * Creates a new <code>ModelMessage</code>.
+     * Creates a new {@code ModelMessage}.
      *
+     * @param messageType The type of this model message.
      * @param id The object identifier.
      * @param source The source of the message. This is what the
      *               message should be associated with.
-     * @param display The <code>FreeColObject</code> to display.
+     * @param display The {@code FreeColObject} to display.
      */
-    public ModelMessage(String id, FreeColGameObject source,
-                        FreeColObject display) {
-        this(MessageType.DEFAULT, id, source, display);
+    public ModelMessage(MessageType messageType, String id,
+                        FreeColGameObject source, FreeColObject display) {
+        this(messageType, id, null, source, display);
     }
 
     /**
-     * Creates a new <code>ModelMessage</code>.
+     * Creates a new {@code ModelMessage}.
      *
      * @param messageType The type of this model message.
      * @param id The object identifier.
@@ -135,57 +162,10 @@ public class ModelMessage extends StringTemplate {
         this(messageType, id, source, getDefaultDisplay(messageType, source));
     }
 
-    /**
-     * Creates a new <code>ModelMessage</code>.
-     *
-     * @param messageType The type of this model message.
-     * @param id The object identifier.
-     * @param defaultId The default identifier.
-     * @param source The source of the message. This is what the
-     *               message should be associated with.
-     */
-    public ModelMessage(MessageType messageType, String id, String defaultId,
-                        FreeColGameObject source) {
-        this(messageType, id, defaultId, source,
-             getDefaultDisplay(messageType, source));
-    }
-
-    /**
-     * Creates a new <code>ModelMessage</code>.
-     *
-     * @param messageType The type of this model message.
-     * @param id The object identifier.
-     * @param source The source of the message. This is what the
-     *               message should be associated with.
-     * @param display The <code>FreeColObject</code> to display.
-     */
-    public ModelMessage(MessageType messageType, String id,
-                        FreeColGameObject source, FreeColObject display) {
-        this(messageType, id, null, source, display);
-    }
-
-    /**
-     * Creates a new <code>ModelMessage</code>.
-     *
-     * @param messageType The type of this model message.
-     * @param id The object identifier.
-     * @param defaultId The default identifier.
-     * @param source The source of the message. This is what the
-     *               message should be associated with.
-     * @param display The <code>FreeColObject</code> to display.
-     */
-    public ModelMessage(MessageType messageType, String id, String defaultId,
-                        FreeColGameObject source, FreeColObject display) {
-        super(id, defaultId, TemplateType.TEMPLATE);
-        this.messageType = messageType;
-        this.sourceId = source.getId();
-        this.displayId = (display != null) ? display.getId() : source.getId();
-    }
-
     /**
      * Creates a new model message by reading a stream.
      *
-     * @param xr The <code>FreeColXMLReader</code> to read from.
+     * @param xr The {@code FreeColXMLReader} to read from.
      * @exception XMLStreamException if there is a problem reading the stream.
      */
     public ModelMessage(FreeColXMLReader xr) throws XMLStreamException {
@@ -193,15 +173,6 @@ public class ModelMessage extends StringTemplate {
         readFromXML(xr);
     }
 
-    /**
-     * Create a new model message by reading a element.
-     *
-     * @param element The <code>Element</code> to read from.
-     */
-    public ModelMessage(Element element) {
-        readFromXMLElement(element);
-    }
-
 
     /**
      * Gets the source of the message.
@@ -268,24 +239,43 @@ public class ModelMessage extends StringTemplate {
         this.messageType = messageType;
     }
 
+    /**
+     * Get the option name for this message.
+     *
+     * @return The option name.
+     */
+    public String getOptionName() {
+        return (this.messageType == null) ? null
+            : this.messageType.getOptionName();
+    }
+
     /**
      * Has this message been displayed?
      *
      * @return True if this message has been displayed.
      */
-    public boolean hasBeenDisplayed() {
-        return beenDisplayed;
+    public boolean getDisplayed() {
+        return this.displayed;
     }
 
     /**
      * Sets whether this message has been displayed.
      *
-     * @param beenDisplayed The new displayed state.
+     * @param displayed The new displayed state.
      */
-    public void setBeenDisplayed(boolean beenDisplayed) {
-        this.beenDisplayed = beenDisplayed;
+    public void setDisplayed(boolean displayed) {
+        this.displayed = displayed;
     }
 
+    /**
+     * Set a default identifier.
+     *
+     * @param id The new default identifier.
+     * @return This {@code ModelMessage}.
+     */
+    public ModelMessage addDefaultId(String id) {
+        return super.setDefaultId(id, ModelMessage.class);
+    }
 
     /**
      * Gets the default display object for the given type.
@@ -338,17 +328,17 @@ public class ModelMessage extends StringTemplate {
     public String getIgnoredMessageKey() {
         switch (getMessageType()) {
         case WAREHOUSE_CAPACITY:
-            String key = getSourceId();
+            StringBuilder sb = new StringBuilder(64);
+            sb.append(getSourceId());
             switch (getTemplateType()) {
             case TEMPLATE:
-                for (String k : getKeys()) {
-                    if ("%goods%".equals(k)) {
-                        key += "-" + getReplacement(k).getId();
-                    }
-                }
+                StringTemplate t = getReplacement("%goods%");
+                if (t != null) sb.append('-').append(t.getId());
+                break;
+            default:
                 break;
             }
-            return key;
+            return sb.toString();
         default:
             break;
         }
@@ -358,7 +348,7 @@ public class ModelMessage extends StringTemplate {
     /**
      * Split a message into a list of text and link objects.
      *
-     * @param player The <code>Player</code> who will see the result.
+     * @param player The {@code Player} who will see the result.
      * @return A list of strings and buttons.
      */
     public List<Object> splitLinks(Player player) {
@@ -370,10 +360,12 @@ public class ModelMessage extends StringTemplate {
         List<Object> result = new ArrayList<>();
         result.add(Messages.message(this));
 
-        for (String key : getKeys()) {
+        for (SimpleEntry<String,StringTemplate> e : entryList()) {
             // Then for each key, check if it can be made into a link.
             // If not, ignore it.
-            String val = Messages.message(getReplacement(key));
+            String key = e.getKey();
+            String val = Messages.message(e.getValue());
+            if (val == null) continue;
             Object b = Utility.getMessageButton(key, val, player, source);
             if (b == null) continue;
 
@@ -388,7 +380,7 @@ public class ModelMessage extends StringTemplate {
                         if (index > start) {
                             next.add(str.substring(start, index));
                         }
-                        next.add(b);
+                        next.add(Utility.getMessageButton(key, val, player, source));
                         start = index + val.length();
                     }
                     next.add(str.substring(start, str.length()));
@@ -401,107 +393,61 @@ public class ModelMessage extends StringTemplate {
         return result;
     }
 
-        
-    // Quasi-override StringTemplate routines to return ModelMessages
+    /**
+     * Get a comparator that sorts on the message source object.
+     *
+     * @param game The {@code Game} to look up source objects in.
+     * @param specialized A map of specialized comparators keyed by class name.
+     * @return The {@code Comparator}.
+     */
+    public static Comparator<ModelMessage> getSourceComparator(final Game game,
+            final java.util.Map<String, Comparator<?>> specialized) {
 
-    /**
-     * {@inheritDoc}
-     */
-    @Override
-    public ModelMessage add(String key, String value) {
-        return (ModelMessage)super.add(key, value);
-    }
+        return new Comparator<ModelMessage>() {
 
-    /**
-     * {@inheritDoc}
-     */
-    @Override
-    public ModelMessage add(String value) {
-        return (ModelMessage)super.add(value);
-    }
-
-    /**
-     * {@inheritDoc}
-     */
-    @Override
-    public ModelMessage addName(String key, String value) {
-        return (ModelMessage)super.addName(key, value);
-    }
-
-    /**
-     * {@inheritDoc}
-     */
-    @Override
-    public ModelMessage addName(String value) {
-        return (ModelMessage)super.addName(value);
-    }
-
-    /**
-     * {@inheritDoc}
-     */
-    @Override
-    public ModelMessage addName(String key, FreeColObject object) {
-        return (ModelMessage)super.addName(key, object);
-    }
-
-    /**
-     * {@inheritDoc}
-     */
-    @Override
-    public ModelMessage addNamed(String key, Named named) {
-        return (ModelMessage)super.addNamed(key, named);
-    }
-    
-    /**
-     * {@inheritDoc}
-     */
-    @Override
-    public ModelMessage addAmount(String key, Number amount) {
-        return (ModelMessage)super.addAmount(key, amount);
-    }
-
-    /**
-     * {@inheritDoc}
-     */
-    @Override
-    public ModelMessage addStringTemplate(String key, StringTemplate template) {
-        return (ModelMessage)super.addStringTemplate(key, template);
-    }
-
-    /**
-     * {@inheritDoc}
-     */
-    @Override
-    public ModelMessage addStringTemplate(StringTemplate template) {
-        return (ModelMessage)super.addStringTemplate(template);
+            /**
+             * {@inheritDoc}
+             */
+            @Override
+            public int compare(ModelMessage message1, ModelMessage message2) {
+                String sourceId1 = message1.getSourceId();
+                String sourceId2 = message2.getSourceId();
+                if (Utils.equals(sourceId1, sourceId2)) {
+                    return messageTypeComparator.compare(message1, message2);
+                }
+                FreeColGameObject source1 = game.getMessageSource(message1);
+                FreeColGameObject source2 = game.getMessageSource(message2);
+                int base = FreeColObject.getObjectClassIndex(source1)
+                    - FreeColObject.getObjectClassIndex(source2);
+                if (base == 0 && specialized != null && source1 != null) {
+                    String name = source1.getClass().getName();
+                    @SuppressWarnings("unchecked")
+                    Comparator<FreeColObject> c
+                        = (Comparator<FreeColObject>)specialized.get(name);
+                    if (c != null) {
+                        base = c.compare(source1, source2);
+                    }
+                }
+                return base;
+            }
+        };
     }
 
 
-    // Override Object
+    // Override FreeColObject
 
     /**
      * {@inheritDoc}
      */
     @Override
-    public boolean equals(Object o) {
-        if (this == o) return true;
-        if (o instanceof ModelMessage) {
-            ModelMessage m = (ModelMessage)o;
-            return sourceId.equals(m.sourceId)
-                && messageType == m.messageType
-                && super.equals(m);
-        }
-        return false;
-    }
-
-    /**
-     * {@inheritDoc}
-     */
-    @Override
-    public int hashCode() {
-        int hash = super.hashCode();
-        hash = 37 * hash + sourceId.hashCode();
-        return 37 * hash + messageType.ordinal();
+    public <T extends FreeColObject> boolean copyIn(T other) {
+        ModelMessage o = copyInCast(other, ModelMessage.class);
+        if (o == null || !super.copyIn(o)) return false;
+        this.sourceId = o.getSourceId();
+        this.displayId = o.getDisplayId();
+        this.messageType = o.getMessageType();
+        this.displayed = o.getDisplayed();
+        return true;
     }
 
 
@@ -526,7 +472,7 @@ public class ModelMessage extends StringTemplate {
 
         xw.writeAttribute(MESSAGE_TYPE_TAG, messageType);
 
-        xw.writeAttribute(HAS_BEEN_DISPLAYED_TAG, beenDisplayed);
+        xw.writeAttribute(HAS_BEEN_DISPLAYED_TAG, displayed);
     }
 
     /**
@@ -543,7 +489,45 @@ public class ModelMessage extends StringTemplate {
         messageType = xr.getAttribute(MESSAGE_TYPE_TAG, 
                                       MessageType.class, MessageType.DEFAULT);
 
-        beenDisplayed = xr.getAttribute(HAS_BEEN_DISPLAYED_TAG, false);
+        displayed = xr.getAttribute(HAS_BEEN_DISPLAYED_TAG, false);
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public String getXMLTagName() { return TAG; }
+
+
+    // Override Object
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public boolean equals(Object o) {
+        if (this == o) return true;
+        if (o instanceof ModelMessage) {
+            ModelMessage other = (ModelMessage)o;
+            return messageType == other.messageType
+                && displayed == other.displayed
+                && Utils.equals(sourceId, other.sourceId)
+                && Utils.equals(displayId, other.displayId)
+                && super.equals(other);
+        }
+        return false;
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public int hashCode() {
+        int hash = super.hashCode();
+        if (sourceId != null) hash = 37 * hash + sourceId.hashCode();
+        if (displayId != null) hash = 37 * hash + displayId.hashCode();
+        hash = 37 * hash + messageType.ordinal();
+        hash = 37 * hash + ((displayed) ? 1 : 0);
+        return hash;
     }
 
     /**
@@ -552,27 +536,13 @@ public class ModelMessage extends StringTemplate {
     @Override
     public String toString() {
         StringBuilder sb = new StringBuilder(128);
-        sb.append("ModelMessage<").append(hashCode())
-            .append(", ").append((sourceId == null) ? "null" : sourceId)
-            .append(", ").append((displayId == null) ? "null" : displayId)
-            .append(", ").append(super.toString())
-            .append(", ").append(messageType)
-            .append(" >");
+        sb.append("[ModelMessage ").append(hashCode())
+            .append(' ').append((sourceId == null) ? "null" : sourceId)
+            .append('/').append((displayId == null) ? "null" : displayId)
+            .append(' ').append(messageType)
+            .append(' ').append(displayed)
+            .append(' ').append(super.toString())
+            .append(']');
         return sb.toString();
     }
-
-    /**
-     * {@inheritDoc}
-     */
-    @Override
-    public String getXMLTagName() { return getXMLElementTagName(); }
-
-    /**
-     * Gets the tag name of the root element representing this object.
-     *
-     * @return "modelMessage"
-     */
-    public static String getXMLElementTagName() {
-        return "modelMessage";
-    }
 }
