@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015   The FreeCol Team
+ *  Copyright (C) 2002-2022   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -22,19 +22,29 @@ package net.sf.freecol.common.model;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
+import java.util.Comparator;
 import java.util.List;
+import java.util.function.Predicate;
+import java.util.stream.Stream;
+
 import javax.xml.stream.XMLStreamException;
 
 import net.sf.freecol.common.io.FreeColXMLReader;
 import net.sf.freecol.common.io.FreeColXMLWriter;
-import net.sf.freecol.common.util.Utils;
 import static net.sf.freecol.common.util.CollectionUtils.*;
+import net.sf.freecol.common.util.Utils;
 
 
 /**
  * This class describes a possible production type of a tile or building.
  */
-public class ProductionType extends FreeColObject {
+public class ProductionType extends FreeColSpecObject {
+
+    public static final String TAG = "production";
+    
+    public static final List<AbstractGoods> EMPTY_LIST
+        = Collections.<AbstractGoods>emptyList();
+
 
     /** Whether this production type applies only to colony center tiles. */
     private boolean unattended;
@@ -55,19 +65,21 @@ public class ProductionType extends FreeColObject {
     /**
      * Simple constructor.
      *
-     * @param specification The <code>Specification</code> to refer to.
+     * @param specification The {@code Specification} to refer to.
      */
     public ProductionType(Specification specification) {
-        setSpecification(specification);
+        super(specification);
     }
 
     /**
      * Creates a new production type that consumes no raw materials
      * and produces the given output.
      *
-     * @param outputs A list of the <code>AbstractGoods</code> produced.
+     * @param outputs A list of the {@code AbstractGoods} produced.
      */
-    public ProductionType(List<AbstractGoods> outputs) {
+    private ProductionType(List<AbstractGoods> outputs) {
+        this((Specification)null);
+
         this.outputs = outputs;
     }
 
@@ -75,54 +87,58 @@ public class ProductionType extends FreeColObject {
      * Creates a new production type that consumes the given raw materials
      * and produces the given output.
      *
-     * @param inputs A list of the <code>AbstractGoods</code> consumed.
-     * @param outputs A list of the <code>AbstractGoods</code> produced.
+     * @param inputs A list of the {@code AbstractGoods} consumed.
+     * @param outputs A list of the {@code AbstractGoods} produced.
      */
     public ProductionType(List<AbstractGoods> inputs,
                           List<AbstractGoods> outputs) {
+        this(outputs);
+
         this.inputs = inputs;
-        this.outputs = outputs;
     }
 
     /**
      * Convenience constructor for a production type with a single output.
      *
-     * @param output The <code>AbstractGoods</code> produced.
+     * @param output The {@code AbstractGoods} produced.
      * @param unattended True if this is unattended production.
      * @param level The difficulty level key.
      */
     public ProductionType(AbstractGoods output, boolean unattended,
                           String level) {
-        this.outputs = new ArrayList<>();
+        this(new ArrayList<AbstractGoods>());
+
         this.outputs.add(output);
         this.unattended = unattended;
         this.productionLevel = level;
     }
 
     /**
-     * Convenience constructor for a new <code>ProductionType</code>
+     * Convenience constructor for a new {@code ProductionType}
      * instance with a single input and output.
      *
-     * @param input The <code>GoodsType</code> consumed.
-     * @param output The <code>GoodsType</code> produced.
+     * @param input The {@code GoodsType} consumed.
+     * @param output The {@code GoodsType} produced.
      * @param amount The amount of goods both produced and consumed.
      */
     public ProductionType(GoodsType input, GoodsType output, int amount) {
+        this((Specification)null);
+        
         if (input != null) {
-            inputs = new ArrayList<>();
-            inputs.add(new AbstractGoods(input, amount));
+            this.inputs = new ArrayList<>();
+            this.inputs.add(new AbstractGoods(input, amount));
         }
         if (output != null) {
-            outputs = new ArrayList<>();
-            outputs.add(new AbstractGoods(output, amount));
+            this.outputs = new ArrayList<>();
+            this.outputs.add(new AbstractGoods(output, amount));
         }
     }
 
     /**
-     * Creates a new <code>ProductionType</code> instance.
+     * Creates a new {@code ProductionType} instance.
      *
-     * @param xr The <code>FreeColXMLReader</code> to read from.
-     * @param specification The <code>Specification</code> to refer to.
+     * @param xr The {@code FreeColXMLReader} to read from.
+     * @param specification The {@code Specification} to refer to.
      * @exception XMLStreamException if there is a problem reading the stream.
      */
     public ProductionType(FreeColXMLReader xr, Specification specification) throws XMLStreamException {
@@ -132,107 +148,13 @@ public class ProductionType extends FreeColObject {
     }
 
 
-    /**
-     * Get the input goods.
-     *
-     * @return A list of the input <code>AbstractGoods</code>.
-     */
-    public final List<AbstractGoods> getInputs() {
-        return (inputs == null) ? Collections.<AbstractGoods>emptyList()
-            : inputs;
-    }
-
-    /**
-     * Set the input goods.
-     *
-     * @param newInputs The new list of input <code>AbstractGoods</code>.
-     */
-    public final void setInputs(final List<AbstractGoods> newInputs) {
-        this.inputs = newInputs;
-    }
-
-    /**
-     * Add a new input.
-     *
-     * @param type The <code>GoodsType</code> to add.
-     * @param amount The amount of goods.
-     */
-    private void addInput(GoodsType type, int amount) {
-        if (inputs == null) inputs = new ArrayList<>(1);
-        inputs.add(new AbstractGoods(type, amount));
-    }
-
-    /**
-     * Get the output goods.
-     *
-     * @return A list of the output <code>AbstractGoods</code>.
-     */
-    public final List<AbstractGoods> getOutputs() {
-        return (outputs == null) ? Collections.<AbstractGoods>emptyList()
-            : outputs;
-    }
-
-    /**
-     * Set the output goods.
-     *
-     * @param newOutputs The new list of output <code>AbstractGoods</code>.
-     */
-    public final void setOutputs(final List<AbstractGoods> newOutputs) {
-        this.outputs = newOutputs;
-    }
-
-    /**
-     * Add a new output.
-     *
-     * @param type The <code>GoodsType</code> to add.
-     * @param amount The amount of goods.
-     */
-    private void addOutput(GoodsType type, int amount) {
-        if (outputs == null) outputs = new ArrayList<>(1);
-        outputs.add(new AbstractGoods(type, amount));
-    }
-
-    /**
-     * Get the goods of the given goods type in this production type.
-     *
-     * @param goodsType The <code>GoodsType</code> to check.
-     * @return The <code>AbstractGoods</code> output if any, otherwise
-     *     null.
-     */
-    public AbstractGoods getOutput(GoodsType goodsType) {
-        if (outputs != null) {
-            AbstractGoods output = AbstractGoods.findByType(goodsType, outputs);
-            if (output != null) return output;
-        }
-        return null;
-    }
-
-    /**
-     * Get the type of the most productive output.
-     *
-     * @return The <code>GoodsType</code> of the most productive output.
-     */
-    public GoodsType getBestOutputType() {
-        AbstractGoods best = null;
-        if (outputs != null) {
-            int amount = 0;
-            for (AbstractGoods output : outputs) {
-                if (amount < output.getAmount()) {
-                    amount = output.getAmount();
-                    best = output;
-                }
-            }
-        }
-        return (best == null) ? null : best.getType();
-    }
-
     /**
      * Get the unattended production state.
      *
      * @return True if this is unattended production.
      */
     public final boolean getUnattended() {
-        return unattended;
+        return this.unattended;
     }
 
     /**
@@ -251,7 +173,7 @@ public class ProductionType extends FreeColObject {
      * @return The production level key.
      */
     public final String getProductionLevel() {
-        return productionLevel;
+        return this.productionLevel;
     }
 
     /**
@@ -262,8 +184,8 @@ public class ProductionType extends FreeColObject {
      */
     public boolean appliesTo(String level) {
         return level == null
-            || productionLevel == null
-            || level.equals(productionLevel);
+            || this.productionLevel == null
+            || level.equals(this.productionLevel);
     }
 
     /**
@@ -274,113 +196,184 @@ public class ProductionType extends FreeColObject {
      * @return True if this production applies.
      */
     public boolean appliesExactly(String level) {
-        return level != null && level.equals(productionLevel);
+        return level != null && level.equals(this.productionLevel);
     }
 
+    /**
+     * Get the input goods list.
+     *
+     * @return A list of the input {@code AbstractGoods}.
+     */
+    public final List<AbstractGoods> getInputList() {
+        return (this.inputs == null) ? EMPTY_LIST : this.inputs;
+    }
+        
+    /**
+     * Get the input goods as a stream.
+     *
+     * @return A stream of the input {@code AbstractGoods}.
+     */
+    public final Stream<AbstractGoods> getInputs() {
+        return (this.inputs == null) ? Stream.<AbstractGoods>empty()
+            : this.inputs.stream();
+    }
+
+    /**
+     * Set the input goods.
+     *
+     * @param newInputs The new list of input {@code AbstractGoods}.
+     */
+    public final void setInputs(final List<AbstractGoods> newInputs) {
+        this.inputs = newInputs;
+    }
+
+    /**
+     * Add a new input.
+     *
+     * @param type The {@code GoodsType} to add.
+     * @param amount The amount of goods.
+     */
+    private void addInput(GoodsType type, int amount) {
+        if (this.inputs == null) this.inputs = new ArrayList<>(1);
+        this.inputs.add(new AbstractGoods(type, amount));
+    }
+
+    /**
+     * Get the output goods list.
+     *
+     * @return A list of the output {@code AbstractGoods}.
+     */
+    public final List<AbstractGoods> getOutputList() {
+        return (this.outputs == null) ? EMPTY_LIST : this.outputs;
+    }
+        
+    /**
+     * Get the output goods as a stream.
+     *
+     * @return A stream of the output {@code AbstractGoods}.
+     */
+    public final Stream<AbstractGoods> getOutputs() {
+        return (this.outputs == null) ? Stream.<AbstractGoods>empty()
+            : this.outputs.stream();
+    }
+
+    /**
+     * Set the output goods.
+     *
+     * @param newOutputs The new list of output {@code AbstractGoods}.
+     */
+    public final void setOutputs(final List<AbstractGoods> newOutputs) {
+        this.outputs = newOutputs;
+    }
+
+    /**
+     * Add a new output.
+     *
+     * @param type The {@code GoodsType} to add.
+     * @param amount The amount of goods.
+     */
+    private void addOutput(GoodsType type, int amount) {
+        if (this.outputs == null) this.outputs = new ArrayList<>(1);
+        this.outputs.add(new AbstractGoods(type, amount));
+    }
+
+    /**
+     * Add a new output.
+     *
+     * @param ag The {@code AbstractGoods} to add.
+     */
+    public void addOutput(AbstractGoods ag) {
+        if (this.outputs == null) this.outputs = new ArrayList<>(1);
+        this.outputs.add(ag);
+    }
+
+    /**
+     * Get the goods of the given goods type in this production type.
+     *
+     * @param goodsType The {@code GoodsType} to check.
+     * @return The {@code AbstractGoods} output if any, otherwise
+     *     null.
+     */
+    public AbstractGoods getOutput(GoodsType goodsType) {
+        return (this.outputs == null) ? null
+            : find(this.outputs, AbstractGoods.matches(goodsType));
+    }
+
+    /**
+     * Get the type of the most productive output.
+     *
+     * @return The {@code GoodsType} of the most productive output.
+     */
+    public GoodsType getBestOutputType() {
+        AbstractGoods goods;
+        return (this.outputs == null
+            || (goods = maximize(this.outputs,
+                    AbstractGoods.ascendingAmountComparator)) == null)
+            ? null
+            : goods.getType();
+    }
 
     /**
      * Convenience function to check if there is an output for a given
      * goods type in a collection of production types.
      *
-     * @param goodsType The <code>GoodsType</code> to use.
-     * @param types A list of <code>ProductionType</code>s to consider.
+     * @param goodsType The {@code GoodsType} to use.
+     * @param types A list of {@code ProductionType}s to consider.
      * @return The most productive output that produces the goods type,
      *     or null if none found.
      */
-    public static boolean canProduce(GoodsType goodsType,
+    public static boolean canProduce(final GoodsType goodsType,
                                      Collection<ProductionType> types) {
-        return any(types, pt -> AbstractGoods.containsType(goodsType,
-                pt.getOutputs()));
+        return any(flatten(types, ProductionType::getOutputs),
+            ag -> goodsType == ag.getType() && ag.getAmount() > 0);
     }
 
     /**
      * Get the production type with the greatest total output of an
      * optional goods type from a collection of production types
      *
-     * @param goodsType An optional <code>GoodsType</code> to restrict the
+     * @param goodsType An optional {@code GoodsType} to restrict the
      *     choice of outputs with.
-     * @param types A collection of <code>ProductionType</code>s to consider.
-     * @return The most productive <code>ProductionType</code>.
+     * @param types A collection of {@code ProductionType}s to consider.
+     * @return The most productive {@code ProductionType}.
      */
     public static ProductionType getBestProductionType(GoodsType goodsType,
         Collection<ProductionType> types) {
-        ProductionType best = null;
-        int bestSum = 0;
-        for (ProductionType pt : types) {
-            int sum = pt.getOutputs().stream()
-                .filter(o -> goodsType == null || goodsType == o.getType())
-                .mapToInt(AbstractGoods::getAmount).sum();
-            if (bestSum < sum) {
-                bestSum = sum;
-                best = pt;
-            }
-        }
-        return best;
+        final Comparator<ProductionType> comp = cachingIntComparator(pt -> {
+                AbstractGoods best = pt.getBestOutputFor(goodsType);
+                return (best == null) ? Integer.MIN_VALUE : best.getAmount();
+            });
+        return maximize(types, comp);
     }
 
     /**
-     * Convenience function to get the best output for a given goods
-     * type from a collection of production types.
+     * Get the output the maximizes production for an optional goods type.
      *
-     * @param goodsType The <code>GoodsType</code> to use.
-     * @param types A collection of <code>ProductionType</code>s to consider.
-     * @return The most productive output that produces the goods type,
-     *     or null if none found.
+     * @param goodsType The optional {@code GoodsType} to check.
+     * @return The best production.
      */
-    public static AbstractGoods getBestOutputFor(GoodsType goodsType,
-                                                 Collection<ProductionType> types) {
-        AbstractGoods best = null;
-        for (ProductionType productionType : types) {
-            for (AbstractGoods output : productionType.getOutputs()) {
-                if (output.getType() == goodsType
-                    && (best == null
-                        || output.getAmount() > best.getAmount())) {
-                    best = output;
-                }
-            }
-        }
-        return best;
+    private AbstractGoods getBestOutputFor(GoodsType goodsType) {
+        final Predicate<AbstractGoods> typePred = ag ->
+            goodsType == null || ag.getType() == goodsType;
+        return maximize(getOutputs(), typePred,
+                        AbstractGoods.ascendingAmountComparator);
     }
 
 
-    // Override Object
+    // Override FreeColObject
 
     /**
      * {@inheritDoc}
      */
     @Override
-    public boolean equals(Object o) {
-        if (this == o) return true;
-        if (o instanceof ProductionType) {
-            ProductionType pt = (ProductionType)o;
-            return super.equals(o)
-                && this.unattended == pt.unattended
-                && Utils.equals(this.productionLevel, pt.productionLevel)
-                && listEquals(this.outputs, pt.outputs)
-                && listEquals(this.inputs, pt.inputs);
-        }
-        return false;
-    }
-
-    /**
-     * {@inheritDoc}
-     */
-    @Override
-    public int hashCode() {
-        int hash = super.hashCode();
-        hash = 31 * hash + ((this.unattended) ? 1 : 0);
-        hash = 31 * hash + Utils.hashCode(this.productionLevel);
-        if (this.outputs != null) {
-            for (AbstractGoods ag : this.outputs) {
-                hash = 31 * hash + Utils.hashCode(ag);
-            }
-        }
-        if (this.inputs != null) {
-            for (AbstractGoods ag : this.inputs) {
-                hash = 31 * hash + Utils.hashCode(ag);
-            }
-        }
-        return hash;
+    public <T extends FreeColObject> boolean copyIn(T other) {
+        ProductionType o = copyInCast(other, ProductionType.class);
+        if (o == null || !super.copyIn(o)) return false;
+        this.unattended = o.getUnattended();
+        this.productionLevel = o.getProductionLevel();
+        this.setOutputs(o.getOutputList());
+        this.setInputs(o.getInputList());
+        return true;
     }
 
 
@@ -488,13 +481,23 @@ public class ProductionType extends FreeColObject {
         if (INPUT_TAG.equals(tag)) {
             GoodsType type = xr.getType(spec, GOODS_TYPE_TAG,
                                         GoodsType.class, (GoodsType)null);
-            addInput(type, xr.getAttribute(VALUE_TAG, -1));
+            if (type == null) {
+                logger.warning("Skipping input with null type: "
+                    + xr.getAttribute(GOODS_TYPE_TAG, (String)null));
+            } else {
+                addInput(type, xr.getAttribute(VALUE_TAG, -1));
+            }
             xr.closeTag(INPUT_TAG);
 
         } else if (OUTPUT_TAG.equals(tag)) {
             GoodsType type = xr.getType(spec, GOODS_TYPE_TAG,
                                         GoodsType.class, (GoodsType)null);
-            addOutput(type, xr.getAttribute(VALUE_TAG, -1));
+            if (type == null) {
+                logger.warning("Skipping output with null type: "
+                    + xr.getAttribute(GOODS_TYPE_TAG, (String)null));
+            } else {
+                addOutput(type, xr.getAttribute(VALUE_TAG, -1));
+            }
             xr.closeTag(OUTPUT_TAG);
 
         } else {
@@ -502,15 +505,61 @@ public class ProductionType extends FreeColObject {
         }
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    public String getXMLTagName() { return TAG; }
+
+
+    // Override Object
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public boolean equals(Object o) {
+        if (this == o) return true;
+        if (o instanceof ProductionType) {
+            ProductionType other = (ProductionType)o;
+            return this.unattended == other.unattended
+                && listEquals(this.outputs, other.outputs)
+                && listEquals(this.inputs, other.inputs)
+                && Utils.equals(this.productionLevel, other.productionLevel)
+                && super.equals(other);
+        }
+        return false;
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public int hashCode() {
+        int hash = super.hashCode();
+        hash = 31 * hash + ((this.unattended) ? 1 : 0);
+        hash = 31 * hash + Utils.hashCode(this.productionLevel);
+        if (this.outputs != null) {
+            for (AbstractGoods ag : this.outputs) {
+                hash = 31 * hash + Utils.hashCode(ag);
+            }
+        }
+        if (this.inputs != null) {
+            for (AbstractGoods ag : this.inputs) {
+                hash = 31 * hash + Utils.hashCode(ag);
+            }
+        }
+        return hash;
+    }
+
     /**
      * {@inheritDoc}
      */
     @Override
     public String toString() {
         StringBuilder result = new StringBuilder(64);
-        result.append("[").append(getId()).append(":");
+        result.append('[').append(getId()).append(':');
         if (productionLevel != null) {
-            result.append(" ").append(productionLevel);
+            result.append(' ').append(productionLevel);
         }
         if (unattended) {
             result.append(" unattended");
@@ -531,22 +580,7 @@ public class ProductionType extends FreeColObject {
             int length = result.length();
             result.replace(length - 2, length, "]");
         }
-        result.append("]");
+        result.append(']');
         return result.toString();
     }
-
-    /**
-     * {@inheritDoc}
-     */
-    @Override
-    public String getXMLTagName() { return getXMLElementTagName(); }
-
-    /**
-     * Gets the tag name of the root element representing this object.
-     *
-     * @return "production".
-     */
-    public static String getXMLElementTagName() {
-        return "production";
-    }
 }
