@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015   The FreeCol Team
+ *  Copyright (C) 2002-2022   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -29,6 +29,7 @@ import javax.xml.stream.XMLStreamException;
 import net.sf.freecol.common.i18n.Messages;
 import net.sf.freecol.common.io.FreeColXMLReader;
 import net.sf.freecol.common.io.FreeColXMLWriter;
+import static net.sf.freecol.common.model.Constants.*;
 import static net.sf.freecol.common.util.CollectionUtils.*;
 import static net.sf.freecol.common.util.StringUtils.*;
 
@@ -36,10 +37,12 @@ import static net.sf.freecol.common.util.StringUtils.*;
 /**
  * A named region on the map.
  */
-public class Region extends FreeColGameObject implements Nameable, Named {
+public class Region extends FreeColGameObject implements Nameable {
 
     private static final Logger logger = Logger.getLogger(Region.class.getName());
 
+    public static final String TAG = "region";
+
     /** The keys for the valid predefined regions. */
     public static final List<String> predefinedRegionKeys
         = makeUnmodifiableList("model.region.arctic", "model.region.antarctic",
@@ -162,18 +165,18 @@ public class Region extends FreeColGameObject implements Nameable, Named {
 
 
     /**
-     * Creates a new <code>Region</code> instance.
+     * Creates a new {@code Region} instance.
      *
-     * @param game The enclosing <code>Game</code>.
+     * @param game The enclosing {@code Game}.
      */
     public Region(Game game) {
         super(game);
     }
 
     /**
-     * Creates a new <code>Region</code> instance.
+     * Creates a new {@code Region} instance.
      *
-     * @param game The enclosing <code>Game</code>.
+     * @param game The enclosing {@code Game}.
      * @param id The object identifier.
      */
     public Region(Game game, String id) {
@@ -218,8 +221,10 @@ public class Region extends FreeColGameObject implements Nameable, Named {
      * @return The i18n-ready name for the region.
      */
     public StringTemplate getLabel() {
-        return (this.key != null) ? StringTemplate.key(getNameKey())
-            : (this.name != null) ? StringTemplate.name(this.name)
+        return (this.key != null)
+            ? StringTemplate.key(Messages.nameKey(this.key))
+            : (this.name != null)
+            ? StringTemplate.name(this.name)
             : StringTemplate.key(type.getUnknownKey());
     }
 
@@ -272,7 +277,7 @@ public class Region extends FreeColGameObject implements Nameable, Named {
     /**
      * Add a child region to this region.
      *
-     * @param child The child <code>Region</code> to add.
+     * @param child The child {@code Region} to add.
      */
     public void addChild(Region child) {
         if (this.children == null) this.children = new ArrayList<>();
@@ -329,19 +334,10 @@ public class Region extends FreeColGameObject implements Nameable, Named {
      *
      * @return The unit identifier, or null if none yet.
      */
-    public synchronized String getDiscoverer() {
+    public final String getDiscoverer() {
         return this.discoverer;
     }
 
-    /**
-     * Set the identifier for the unit the discovered the region.
-     *
-     * @param discoverer The unit identifier to set.
-     */
-    public synchronized void setDiscoverer(String discoverer) {
-        this.discoverer = discoverer;
-    }
-
     /**
      * Gets a discoverable Region or null.  If this region is
      * discoverable, it is returned.  If not, a discoverable parent is
@@ -378,7 +374,7 @@ public class Region extends FreeColGameObject implements Nameable, Named {
     /**
      * Gets the player that discovered the region.
      *
-     * @return The discovering <code>Player</code>.
+     * @return The discovering {@code Player}.
      */
     public final Player getDiscoveredBy() {
         return this.discoveredBy;
@@ -411,26 +407,56 @@ public class Region extends FreeColGameObject implements Nameable, Named {
         this.scoreValue = newScoreValue;
     }
 
+    /**
+     * Check if this region is has been discovered.
+     *
+     * Reserve the ability to discover this region by setting the discoverer
+     * field.
+     * Called in csCheckDiscoverRegion when the unit moves into a
+     * discoverable region.
+     *
+     * @param unit The {@code Unit} that might have discovered the region.
+     * @return True if the region has been discovered.
+     */
+    public synchronized boolean checkDiscover(Unit unit) {
+        if (this.discoverable) {
+            if (this.discoverer == null) {
+                this.discoverer = unit.getId();
+                return true;
+            } else if (this.discoverer.equals(unit.getId())) {
+                return true; // Work around a desynch problem
+            }
+        }
+        return false;
+    }
+
     /**
      * Discover this region (and its children).
      *
-     * @param player The discovering <code>Player</code>.
-     * @param turn The <code>Turn</code> of discovery.
-     * @return A list of discovered <code>Region</code>s.
+     * @param player The discovering {@code Player}.
+     * @param unit The discovering {@code Unit}.
+     * @param turn The {@code Turn} of discovery.
+     * @return A list of discovered {@code Region}s.
      */
-    public List<Region> discover(Player player, Turn turn) {
-        List<Region> result = new ArrayList<>();
+    public List<Region> discover(Player player, Unit unit, Turn turn) {
+        if (unit == null || this.discoverer == null ||
+            !this.discoverer.equals(unit.getId())) {
+            // checkDiscover must have been run
+            return Collections.<Region>emptyList();
+        }
         this.discoveredBy = player;
         this.discoveredIn = turn;
         this.discoverable = false;
-        result.add(this);
-        for (Region r : getChildren()) {
-            if (!r.getDiscoverable()) continue;
+        List<Region> discov = transform(getChildren(), Region::getDiscoverable);
+        for (Region r : discov) {
             r.discoveredBy = player;
+            r.discoverer = unit.getId();
             r.discoveredIn = turn;
             r.discoverable = false;
-            result.add(r);
         }
+        List<Region> result = new ArrayList<>(discov.size()+1);
+        result.add(this);
+        result.addAll(discov);
         return result;
     }
 
@@ -438,6 +464,7 @@ public class Region extends FreeColGameObject implements Nameable, Named {
     /**
      * Is a key one of the dodgy keys that were generated up to 0.11.3?
      *
+     * @param key The key to check.
      * @return A valid key or null if already null or invalid.
      */
     private String fixRegionKey(String key) {
@@ -473,14 +500,28 @@ public class Region extends FreeColGameObject implements Nameable, Named {
     }
 
 
-    // Implement Named
-    
+    // Override FreeColObject
+
     /**
      * {@inheritDoc}
      */
     @Override
-    public final String getNameKey() {
-        return Messages.nameKey(this.key);
+    public <T extends FreeColObject> boolean copyIn(T other) {
+        Region o = copyInCast(other, Region.class);
+        if (o == null || !super.copyIn(o)) return false;
+        final Game game = getGame();
+        this.name = o.getName();
+        this.key = o.getKey();
+        this.type = o.getType();
+        this.parent = game.updateRef(o.getParent());
+        this.children = game.updateRef(o.getChildren());
+        this.claimable = o.getClaimable();
+        this.discoverable = o.getDiscoverable();
+        this.discoveredIn = o.getDiscoveredIn();
+        this.discoveredBy = game.updateRef(o.getDiscoveredBy());
+        this.discoverer = o.getDiscoverer();
+        this.scoreValue = o.getScoreValue();
+        return true;
     }
 
 
@@ -587,8 +628,7 @@ public class Region extends FreeColGameObject implements Nameable, Named {
         discoveredBy = xr.findFreeColGameObject(getGame(), DISCOVERED_BY_TAG,
             Player.class, (Player)null, false);
 
-        parent = xr.makeFreeColGameObject(getGame(), PARENT_TAG,
-                                          Region.class, false);
+        parent = xr.makeFreeColObject(getGame(), PARENT_TAG, Region.class, false);
     }
 
     /**
@@ -610,8 +650,8 @@ public class Region extends FreeColGameObject implements Nameable, Named {
         final String tag = xr.getLocalName();
 
         if (CHILD_TAG.equals(tag)) {
-            addChild(xr.makeFreeColGameObject(getGame(), ID_ATTRIBUTE_TAG,
-                                              Region.class, true));
+            addChild(xr.makeFreeColObject(getGame(), ID_ATTRIBUTE_TAG,
+                                          Region.class, true));
             xr.closeTag(CHILD_TAG);
         
         } else {
@@ -622,30 +662,27 @@ public class Region extends FreeColGameObject implements Nameable, Named {
     /**
      * {@inheritDoc}
      */
-    @Override
-    public String toString() {
-        StringBuilder sb = new StringBuilder(32);
-        sb.append("[").append(getId())
-            .append(" ").append((key != null) ? key : (name != null) ? name
-                : "<unnamed>")
-            .append(" ").append(type);
-        if (getDiscoverable()) sb.append("!");
-        sb.append("]");
-        return sb.toString();
-    }
+    public String getXMLTagName() { return TAG; }
+
+
+    // Override Object
 
     /**
      * {@inheritDoc}
      */
     @Override
-    public String getXMLTagName() { return getXMLElementTagName(); }
-
-    /**
-     * Gets the tag name of the root element representing this object.
-     *
-     * @return "region".
-     */
-    public static String getXMLElementTagName() {
-        return "region";
+    public String toString() {
+        StringBuilder sb = new StringBuilder(32);
+        sb.append('[').append(getId())
+            .append(' ').append((key != null) ? key : (name != null) ? name
+                : "<unnamed>")
+            .append(' ').append(type);
+        if (getDiscoverable()) {
+            sb.append('!');
+        } else {
+            sb.append("discoverer=").append(this.discoverer);
+        }
+        sb.append(']');
+        return sb.toString();
     }
 }
