@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015   The FreeCol Team
+ *  Copyright (C) 2002-2022   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -19,20 +19,23 @@
 
 package net.sf.freecol.common.model;
 
+import java.util.AbstractMap.SimpleEntry;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
+import java.util.logging.Level;
 
 import javax.xml.stream.XMLStreamException;
 
 import net.sf.freecol.common.i18n.Messages;
 import net.sf.freecol.common.io.FreeColXMLReader;
 import net.sf.freecol.common.io.FreeColXMLWriter;
+import static net.sf.freecol.common.util.CollectionUtils.*;
 import net.sf.freecol.common.util.Utils;
 
 
 /**
- * The <code>StringTemplate</code> represents a non-localized string
+ * The {@code StringTemplate} represents a non-localized string
  * that can be localized by looking up its value in a message bundle
  * or similar Map. The StringTemplate may contain variables (keys)
  * delimited by the '%' character, such as "%amount%" that will be
@@ -46,6 +49,12 @@ import net.sf.freecol.common.util.Utils;
 
 public class StringTemplate extends FreeColObject {
 
+    public static final String TAG = "stringTemplate";
+
+    /** Fixed trivial return value for entryList(). */
+    private static final List<SimpleEntry<String,StringTemplate>> emptyList
+        = Collections.<SimpleEntry<String,StringTemplate>>emptyList();
+
     /**
      * The type of this StringTemplate, either NAME, a proper name
      * that must not be localized (e.g. "George Washington"), or KEY,
@@ -65,52 +74,47 @@ public class StringTemplate extends FreeColObject {
      */
     private String defaultId = null;
 
-    /** The keys to replace within the string template. */
-    private List<String> keys = null;
-
-    /** The values with which to replace the keys in the string template. */
-    private List<StringTemplate> replacements = null;
+    /** The key,value paris to apply within the string template. */
+    private List<SimpleEntry<String,StringTemplate>> kv = null;
 
 
     /**
-     * Deliberately empty constructor.
+     * Trivial constructor to allow creation with Game.newInstance.
      */
-    protected StringTemplate() {}
+    public StringTemplate() {}
 
     /**
      * Copy an existing template, but with a new identifier.
      *
      * @param id The object identifier.
-     * @param template A <code>StringTemplate</code> to copy.
+     * @param template A {@code StringTemplate} to copy.
      */
     protected StringTemplate(String id, StringTemplate template) {
         setId(id);
         this.templateType = template.templateType;
         this.defaultId = template.defaultId;
-        this.keys = template.keys;
-        this.replacements = template.replacements;
+        this.kv = template.kv;
     }
 
     /**
-     * Creates a new <code>StringTemplate</code> instance.
+     * Creates a new {@code StringTemplate} instance.
      *
      * @param id The object identifier.
      * @param defaultId The default identifier.
-     * @param templateType The <code>TemplateType</code> for this template.
+     * @param templateType The {@code TemplateType} for this template.
      */
     protected StringTemplate(String id, String defaultId,
                              TemplateType templateType) {
         setId(id);
         this.defaultId = defaultId;
         this.templateType = templateType;
-        this.keys = null;
-        this.replacements = null;
+        this.kv = null;
     }
 
     /**
-     * Create a new <code>StringTemplate</code> by reading a stream.
+     * Create a new {@code StringTemplate} by reading a stream.
      *
-     * @param xr The <code>FreeColXMLReader</code> to read.
+     * @param xr The {@code FreeColXMLReader} to read.
      * @exception XMLStreamException if there is a problem reading the stream.
      */
     public StringTemplate(FreeColXMLReader xr) throws XMLStreamException {
@@ -125,9 +129,6 @@ public class StringTemplate extends FreeColObject {
     }
 
     public static StringTemplate name(String value) {
-        if (value == null) {
-            logger.warning("NULL NAME TEMPLATE\n" + net.sf.freecol.common.debug.FreeColDebugger.stackTraceToString());
-        }
         return new StringTemplate(value, null, TemplateType.NAME);
     }
 
@@ -173,69 +174,119 @@ public class StringTemplate extends FreeColObject {
     /**
      * Set the default identifier.
      *
-     * @param newDefaultId The new default identifier
+     * @param id The new default identifier.
      */
-    public final void setDefaultId(final String newDefaultId) {
-        this.defaultId = newDefaultId;
+    public final void setDefaultId(String id) {
+        this.defaultId = id;
     }
 
     /**
-     * Get the keys.
+     * Wrapper for subclasses to set the default identifier and return the
+     * setting object.
      *
-     * @return A list of keys.
+     * @param <T> The actual return type.
+     * @param id The new default identifier
+     * @param returnClass The expected return class.
+     * @return The setting object.
      */
-    public final List<String> getKeys() {
-        return (this.keys == null) ? Collections.<String>emptyList()
-            : this.keys;
+    protected <T extends StringTemplate> T setDefaultId(final String id,
+                                                        Class<T> returnClass) {
+        setDefaultId(id);
+        try {
+            return returnClass.cast(this);
+        } catch (ClassCastException cce) {
+            logger.log(Level.WARNING, "Invalid class "
+                + this.getClass() + " referenced.", cce);
+        }
+        return null;            
     }
 
+    /**
+     * Has nothing been added to this template?
+     *
+     * @return True if the template is empty.
+     */
+    public boolean isEmpty() {
+        return this.kv == null || this.kv.isEmpty();
+    }
+
+    /**
+     * Get the list of key, value pairs.
+     *
+     * @return The pairs.
+     */
+    public List<SimpleEntry<String,StringTemplate>> entryList() {
+        return (this.kv != null) ? this.kv : emptyList;
+    }
+
+    /**
+     * Try to find the replacement for a given key.
+     *
+     * @param key The key to look for.
+     * @return The value found, otherwise null.
+     */
+    public StringTemplate getReplacement(String key) {
+        if (this.kv == null) return null;
+        SimpleEntry<String,StringTemplate> val
+            = find(this.kv, matchKeyEquals(key, SimpleEntry::getKey));
+        return (val == null) ? null : val.getValue();
+    }
+
+    /**
+     * Add a key, value pair.
+     *
+     * @param key The {@code String} key.
+     * @param value The {@code StringTemplate} value.
+     */
+    private void addPair(String key, StringTemplate value) {
+        if (key == null && value == null) {
+            throw new RuntimeException("Null key and pair: " + this);
+        }
+        if (this.kv == null) this.kv = new ArrayList<>();
+        this.kv.add(new SimpleEntry<>(key, value));
+    }
+
+    // @compat 0.11.x
     /**
      * Add a key.
      * 
      * @param key The key to add.
      */
     private void addKey(String key) {
-        if (this.keys == null) this.keys = new ArrayList<>();
-        this.keys.add(key);
+        addPair(key, null);
     }
 
-    /**
-     * Get the replacements.
-     *
-     * @return A list of replacements.
-     */
-    public final List<StringTemplate> getReplacements() {
-        return (this.replacements == null)
-            ? Collections.<StringTemplate>emptyList()
-            : this.replacements;
-    }
-    
     /**
      * Add a replacement.
      *
-     * @param replacement The <code>StringTemplate</code> replacement to add.
+     * @param replacement The {@code StringTemplate} replacement to add.
      */
     private void addReplacement(StringTemplate replacement) {
-        if (this.replacements == null) this.replacements = new ArrayList<>();
-        this.replacements.add(replacement);
-    }
-
-    /**
-     * Get the replacement value for a given key.
-     *
-     * @param key The key to find a replacement for.
-     * @return The replacement found, or null if none found.
-     */
-    public final StringTemplate getReplacement(String key) {
-        if (this.keys != null && this.replacements != null) {
-            for (int index = 0; index < this.keys.size(); index++) {
-                if (key.equals(this.keys.get(index))) {
-                    return (this.replacements.size() <= index) ? null
-                        : this.replacements.get(index);
-                }
+        if (this.kv == null) this.kv = new ArrayList<>();
+        for (SimpleEntry<String,StringTemplate> e : this.kv) {
+            if (e.getValue() == null) {
+                e.setValue(replacement);
+                return;
             }
         }
-        return null;
+        addPair(null, replacement);
+    }
+    // end @compat 0.11.x
+
+    /**
+     * Add an optional key and replacement.  Helper function for the add*()
+     * routines that follow.
+     *
+     * @param <T> The actual return type.
+     * @param key The optional key.
+     * @param value The replacement {@code StringTemplate}.
+     * @return This object, cast back to its original class.
+     */
+    @SuppressWarnings("unchecked")
+    private final <T extends StringTemplate> T complete(String key,
+                                                        StringTemplate value) {
+        addPair(key, value);
+        return (T)this;
     }
 
     /**
@@ -243,18 +294,17 @@ public class StringTemplate extends FreeColObject {
      *
      * This is only possible if the template is of type TEMPLATE.
      *
+     * @param <T> The actual return type.
      * @param key The key to add.
      * @param value The corresponding replacement.
-     * @return This <code>StringTemplate</code>.
+     * @return This.
      */
-    public StringTemplate add(String key, String value) {
+    public <T extends StringTemplate> T add(String key, String value) {
         if (this.templateType != TemplateType.TEMPLATE) {
-            throw new IllegalArgumentException("Cannot add key-value pair"
+            throw new RuntimeException("Cannot add key-value pair"
                 + " to StringTemplate." + this.templateType);
         }
-        addKey(key);
-        addReplacement(this.key(value));
-        return this;
+        return complete(key, StringTemplate.key(value));
     }
 
     /**
@@ -262,16 +312,16 @@ public class StringTemplate extends FreeColObject {
      *
      * This is only possible if the template is of type LABEL.
      *
+     * @param <T> The actual return type.
      * @param value The replacement value.
-     * @return This <code>StringTemplate</code>.
+     * @return This.
      */
-    public StringTemplate add(String value) {
+    public <T extends StringTemplate> T add(String value) {
         if (this.templateType != TemplateType.LABEL) {
-            throw new IllegalArgumentException("Cannot add a single string"
+            throw new RuntimeException("Cannot add a single string"
                 + " to StringTemplate." + this.templateType);
         }
-        addReplacement(this.key(value));
-        return this;
+        return complete(null, StringTemplate.key(value));
     }
 
     /**
@@ -279,18 +329,17 @@ public class StringTemplate extends FreeColObject {
      *
      * This is only possible if the template is of type TEMPLATE.
      *
+     * @param <T> The actual return type.
      * @param key The key to add.
      * @param value The corresponding replacement.
-     * @return This <code>StringTemplate</code>.
+     * @return This.
      */
-    public StringTemplate addName(String key, String value) {
+    public <T extends StringTemplate> T addName(String key, String value) {
         if (this.templateType != TemplateType.TEMPLATE) {
-            throw new IllegalArgumentException("Cannot add key-name pair"
+            throw new RuntimeException("Cannot add key-name pair"
                 + " to StringTemplate." + this.templateType);
         }
-        addKey(key);
-        addReplacement(this.name(value));
-        return this;
+        return complete(key, StringTemplate.name(value));
     }
 
     /**
@@ -298,18 +347,17 @@ public class StringTemplate extends FreeColObject {
      *
      * This is only possible if the StringTemplate is of type TEMPLATE.
      *
+     * @param <T> The actual return type.
      * @param key The key to add.
-     * @param object The replacement <code>FreeColObject</code>.
-     * @return This <code>StringTemplate</code>.
+     * @param object The replacement {@code FreeColObject}.
+     * @return This.
      */
-    public StringTemplate addName(String key, FreeColObject object) {
+    public <T extends StringTemplate> T addName(String key, FreeColObject object) {
         if (this.templateType != TemplateType.TEMPLATE) {
-            throw new IllegalArgumentException("Cannot add key-object pair"
+            throw new RuntimeException("Cannot add key-object pair"
                 + " to StringTemplate." + this.templateType);
         }
-        addKey(key);
-        addReplacement(this.key(Messages.nameKey(object.getId())));
-        return this;
+        return complete(key, StringTemplate.key(Messages.nameKey(object.getId())));
     }
 
     /**
@@ -317,47 +365,50 @@ public class StringTemplate extends FreeColObject {
      *
      * This is only possible if the StringTemplate is of type LABEL.
      *
+     * @param <T> The actual return type.
      * @param value The replacement value.
-     * @return This <code>StringTemplate</code>.
+     * @return This.
      */
-    public StringTemplate addName(String value) {
+    public <T extends StringTemplate> T addName(String value) {
         if (this.templateType != TemplateType.LABEL) {
-            throw new IllegalArgumentException("Cannot add a single string"
+            throw new RuntimeException("Cannot add a single string"
                 + " to StringTemplate." + this.templateType);
         }
-        addReplacement(this.name(value));
-        return this;
+        return complete(null, StringTemplate.name(value));
     }
 
     /**
      * Add a key and named object to this template.
      *
+     * @param <T> The actual return type.
      * @param key The key to add.
-     * @param named The <code>Named</code> to add.
-     * @return This <code>StringTemplate</code>.
+     * @param named The {@code Named} to add.
+     * @return This.
      */
-    public StringTemplate addNamed(String key, Named named) {
+    public <T extends StringTemplate> T addNamed(String key, Named named) {
         return add(key, named.getNameKey());
     }
 
     /**
      * Add named object without key to this template.
      *
-     * @param named The <code>Named</code> to add.
-     * @return This <code>StringTemplate</code>.
+     * @param <T> The actual return type.
+     * @param named The {@code Named} to add.
+     * @return This.
      */
-    public StringTemplate addNamed(Named named) {
+    public <T extends StringTemplate> T addNamed(Named named) {
         return add(named.getNameKey());
     }
 
     /**
      * Add a key and an integer value to replace it to this template.
      *
+     * @param <T> The actual return type.
      * @param key The key to add.
-     * @param amount The <code>Number</code> value to add.
-     * @return This <code>StringTemplate</code>.
+     * @param amount The {@code Number} value to add.
+     * @return This.
      */
-    public StringTemplate addAmount(String key, Number amount) {
+    public <T extends StringTemplate> T addAmount(String key, Number amount) {
         return addName(key, amount.toString());
     }
 
@@ -366,19 +417,18 @@ public class StringTemplate extends FreeColObject {
      *
      * This is only possible if the StringTemplate is of type TEMPLATE.
      *
+     * @param <T> The actual return type.
      * @param key The key to add.
-     * @param template The <code>StringTemplate</code> value.
-     * @return This <code>StringTemplate</code>.
+     * @param template The {@code StringTemplate} value.
+     * @return This.
      */
-    public StringTemplate addStringTemplate(String key,
-                                            StringTemplate template) {
+    public <T extends StringTemplate> T addStringTemplate(String key,
+                                                          StringTemplate template) {
         if (this.templateType != TemplateType.TEMPLATE) {
-            throw new IllegalArgumentException("Cannot add key-template pair"
+            throw new RuntimeException("Cannot add key-template pair"
                 + " to StringTemplate." + this.templateType);
         }
-        addKey(key);
-        addReplacement(template);
-        return this;
+        return complete(key, template);
     }
 
     /**
@@ -386,97 +436,58 @@ public class StringTemplate extends FreeColObject {
      *
      * This is only possible if the StringTemplate is of type LABEL.
      *
-     * @param template The replacement <code>StringTemplate</code>.
-     * @return This <code>StringTemplate</code>.
+     * @param <T> The actual return type.
+     * @param template The replacement {@code StringTemplate}.
+     * @return This.
      */
-    public StringTemplate addStringTemplate(StringTemplate template) {
+    public <T extends StringTemplate> T addStringTemplate(StringTemplate template) {
         if (this.templateType != TemplateType.LABEL) {
-            throw new IllegalArgumentException("Cannot add a template"
+            throw new RuntimeException("Cannot add a template"
                 + " to StringTemplate." + this.templateType);
         }
-        addReplacement(template);
-        return this;
+        return complete(null, template);
+    }
+
+    /**
+     * Add a tagged value.
+     *
+     * Functionally identical to add(), but used to distinguish the special
+     * cases at the point they are used.
+     *
+     * @param <T> The actual return type.
+     * @param key The tag.
+     * @param value The special tag value.
+     * @return This.
+     */
+    public <T extends StringTemplate> T addTagged(String key, String value) {
+        return add(key, value);
     }
 
 
-    // Override Object
+    // Overide FreeColObject
 
     /**
      * {@inheritDoc}
      */
     @Override
-    public boolean equals(Object o) {
-        if (this == o) return true;
-        if (o instanceof StringTemplate) {
-            StringTemplate t = (StringTemplate)o;
-            if (!super.equals(o)
-                || this.templateType != t.templateType
-                || !Utils.equals(this.defaultId, t.defaultId)) return false;
-            switch (this.templateType) {
-            case TEMPLATE:
-                if ((this.keys == null) != (t.keys == null))
-                    return false;
-                if (this.keys != null) {
-                    if (this.keys.size() != t.keys.size()
-                        || this.keys.size() != this.replacements.size())
-                        return false;
-                    for (int i = 0; i < this.keys.size(); i++) {
-                        if (!this.keys.get(i)
-                            .equals(t.keys.get(i))) return false;
-                    }
-                }
-                // Fall through
-            case LABEL:
-                if ((this.replacements == null) != (t.replacements == null))
-                    return false;
-                if (this.replacements != null) {
-                    if (this.replacements.size() != t.replacements.size())
-                        return false;
-                    for (int i = 0; i < this.replacements.size(); i++) {
-                        if (!this.replacements.get(i)
-                            .equals(t.replacements.get(i))) return false;
-                    }
-                }
-                break;
-            default:
-                break;
-            }
-            return true;
-        }
-        return false;
-    }
-
-    /**
-     * {@inheritDoc}
-     */
-    @Override
-    public int hashCode() {
-        int hash = super.hashCode();
-        hash = 31 * hash + this.templateType.ordinal();
-        hash = 31 * hash + Utils.hashCode(this.defaultId);
-        switch (this.templateType) {
-        case TEMPLATE:
-            for (String key : getKeys()) {
-                hash = 31 * hash + Utils.hashCode(key);
-            }
-            // Fall through
-        case LABEL:
-            for (StringTemplate replacement : getReplacements()) {
-                hash = 31 * hash + Utils.hashCode(replacement);
-            }
-            break;
-        default:
-            break;
-        }
-        return hash;
+    public <T extends FreeColObject> boolean copyIn(T other) {
+        StringTemplate o = copyInCast(other, StringTemplate.class);
+        if (o == null || !super.copyIn(o)) return false;
+        this.templateType = o.getTemplateType();
+        this.defaultId = o.getDefaultId();
+        this.kv = o.kv;
+        return true;
     }
 
 
     // Serialization
 
     private static final String DEFAULT_ID_TAG = "defaultId";
-    private static final String KEY_TAG = "key";
+    private static final String PAIR_TAG = "pair";
     private static final String TEMPLATE_TYPE_TAG = "templateType";
+    // @compat 0.11.x
+    private static final String OLD_KEY_TAG = "key";
+    // end @compat 0.11.x
 
 
     /**
@@ -500,21 +511,18 @@ public class StringTemplate extends FreeColObject {
     protected void writeChildren(FreeColXMLWriter xw) throws XMLStreamException {
         super.writeChildren(xw);
 
-        if (keys != null) {
-            for (String key : keys) {
-                xw.writeStartElement(KEY_TAG);
+        if (this.kv != null) {
+            for (SimpleEntry<String,StringTemplate> e : this.kv) {
+                xw.writeStartElement(PAIR_TAG);
+                
+                String key = e.getKey(); // OK if null
+                if (key != null) xw.writeAttribute(VALUE_TAG, key);
 
-                xw.writeAttribute(VALUE_TAG, key);
+                e.getValue().toXML(xw); // value is always present
 
                 xw.writeEndElement();
             }
         }
-
-        if (replacements != null) {
-            for (StringTemplate replacement : replacements) {
-                replacement.toXML(xw);
-            }
-        }
     }
 
     /**
@@ -523,7 +531,7 @@ public class StringTemplate extends FreeColObject {
     @Override
     protected void readAttributes(FreeColXMLReader xr) throws XMLStreamException {
         super.readAttributes(xr);
-
+        
         templateType = xr.getAttribute(TEMPLATE_TYPE_TAG,
             TemplateType.class, TemplateType.TEMPLATE);
 
@@ -536,8 +544,7 @@ public class StringTemplate extends FreeColObject {
     @Override
     protected void readChildren(FreeColXMLReader xr) throws XMLStreamException {
         // Clear containers.
-        if (keys != null) keys.clear();
-        if (replacements != null) replacements.clear();
+        if (kv != null) kv.clear();
 
         super.readChildren(xr);
     }
@@ -549,75 +556,132 @@ public class StringTemplate extends FreeColObject {
     protected void readChild(FreeColXMLReader xr) throws XMLStreamException {
         final String tag = xr.getLocalName();
 
-        if (KEY_TAG.equals(tag)) {
-            addKey(xr.getAttribute(VALUE_TAG, (String)null));
-            xr.closeTag(KEY_TAG);
+        if (PAIR_TAG.equals(tag)) {
+            // key == null is allowed
+            String key = xr.getAttribute(VALUE_TAG, (String)null);
+            StringTemplate val = null;
+            while (xr.moreTags()) {
+                final String inner = xr.getLocalName();
+                if (StringTemplate.TAG.equals(inner)) {
+                    if (val == null) {
+                        val = new StringTemplate(xr);
+                    } else {
+                        xr.expectTag(PAIR_TAG);
+                    }
+                } else {
+                    xr.expectTag(StringTemplate.TAG);
+                }
+            }
+            addPair(key, val);
 
-        } else if (StringTemplate.getXMLElementTagName().equals(tag)) {
+        // @compat 0.11.x
+        // old format where key and value were separated
+        } else if (OLD_KEY_TAG.equals(tag)) {
+            addKey(xr.getAttribute(VALUE_TAG, (String)null));
+            xr.closeTag(OLD_KEY_TAG);
+
+        } else if (StringTemplate.TAG.equals(tag)) {
             addReplacement(new StringTemplate(xr));
-        
+        // end @compat 0.11.x
+
         } else {
             super.readChild(xr);
         }
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    public String getXMLTagName() { return TAG; }
+
+
+    // Override Object
+
     /**
      * {@inheritDoc}
      */
     @Override
-    public String toString() {
-        StringBuilder sb = new StringBuilder(64);
-        sb.append(templateType).append(": ");
-        switch (templateType) {
-        case LABEL:
-            if (this.replacements == null) {
-                sb.append(getId());
-            } else {
-                for (StringTemplate object : this.replacements) {
-                    sb.append(object).append(getId());
+    public boolean equals(Object o) {
+        if (this == o) return true;
+        if (o instanceof StringTemplate) {
+            StringTemplate other = (StringTemplate)o;
+            int i;
+            if (this.templateType != other.templateType
+                || !Utils.equals(this.defaultId, other.defaultId))
+                return false;
+            if ((this.kv == null) != (other.kv == null)) return false;
+            if (this.kv != null) {
+                if (this.kv.size() != other.kv.size()) return false;
+                i = 0;
+                for (SimpleEntry<String,StringTemplate> e : this.kv) {
+                    if (!e.getKey().equals(other.kv.get(i).getKey()))
+                        return false;
+                    if (!e.getValue().equals(other.kv.get(i).getValue()))
+                        return false;
+                    i++;
                 }
             }
-            break;
-        case TEMPLATE:
-            sb.append(getId());
-            if (this.defaultId != null) {
-                sb.append(" (").append(this.defaultId).append(")");
-            }
-            sb.append(" [");
-            if (this.keys != null) {
-                for (int index = 0; index < this.keys.size(); index++) {
-                    sb.append("[").append(this.keys.get(index)).append(": ")
-                        .append(this.replacements.get(index)).append("]");
-                }
-            }
-            sb.append("]");
-            break;
-        case KEY:
-            sb.append(getId());
-            if (this.defaultId != null) {
-                sb.append(" (").append(this.defaultId).append(")");
-            }
-            break;
-        case NAME:
-        default:
-            sb.append(getId());
-            break;
+            return super.equals(other);
         }
-        return sb.toString();
+        return false;
     }
 
     /**
      * {@inheritDoc}
      */
     @Override
-    public String getXMLTagName() { return getXMLElementTagName(); }
+    public int hashCode() {
+        int hash = super.hashCode();
+        hash = 31 * hash + this.templateType.ordinal();
+        hash = 31 * hash + Utils.hashCode(this.defaultId);
+        if (this.kv != null) {
+            for (SimpleEntry<String,StringTemplate> e : this.kv) {
+                hash = 31 * hash + Utils.hashCode(e.getKey());
+                hash = 31 * hash + Utils.hashCode(e.getValue());
+            }
+        }
+        return hash;
+    }
 
     /**
-     * Gets the tag name of the root element representing this object.
-     *
-     * @return "stringTemplate".
+     * {@inheritDoc}
      */
-    public static String getXMLElementTagName() {
-        return "stringTemplate";
+    @Override
+    public String toString() {
+        StringBuilder sb = new StringBuilder(64);
+        sb.append(templateType).append(':').append(getId());
+        if (this.defaultId != null) {
+            sb.append('(').append(this.defaultId).append(')');
+        }
+        String start;
+        switch (templateType) {
+        case LABEL:
+            if (this.kv != null) {
+                int index = 0;
+                start = "[";
+                for (SimpleEntry<String,StringTemplate> e : this.kv) {
+                    sb.append(start).append(index++)
+                        .append(':').append(e.getValue());
+                    start = " ";
+                }
+                sb.append(']');
+            }
+            break;
+        case TEMPLATE:
+            sb.append('[');
+            if (this.kv != null) {
+                start = "";
+                for (SimpleEntry<String,StringTemplate> e : this.kv) {
+                    sb.append(start).append(e.getKey())
+                        .append('=').append(e.getValue());
+                    start = "";
+                }
+            }
+            sb.append(']');
+            break;
+        case KEY: case NAME: default:
+            break;
+        }
+        return sb.toString();
     }
 }
