@@ -46,6 +46,9 @@ public final class TileType extends FreeColSpecObjectType
     public static final String TAG = "tile-type";
 
     public static enum RangeType { HUMIDITY, TEMPERATURE, ALTITUDE };
+    
+    public static final int HILLS_ALTITUDE = 15;
+    public static final int MOUNTAINS_ALTITUDE = 25;
 
     /**
      * Use these tile types only for "land maps", i.e. maps that only
@@ -188,6 +191,22 @@ public final class TileType extends FreeColSpecObjectType
     public int getBasicMoveCost() {
         return basicMoveCost;
     }
+    
+    /**
+     * Checks if this {@code TileType} should be considered a hill when
+     * generating the map.
+     */
+    public boolean isHills() {
+        return altitude[0] <= HILLS_ALTITUDE && altitude[1] >= HILLS_ALTITUDE;
+    }
+
+    /**
+     * Checks if this {@code TileType} should be considered a mountain when
+     * generating the map.
+     */
+    public boolean isMountains() {
+        return altitude[0] <= MOUNTAINS_ALTITUDE && altitude[1] >= MOUNTAINS_ALTITUDE;
+    }
 
     /**
      * Gets the basic work turns to build an improvement on this tile type.
@@ -426,7 +445,15 @@ public final class TileType extends FreeColSpecObjectType
                                       UnitType unitType) {
         if (goodsType == null) return 0;
         int amount = getBaseProduction(null, goodsType, unitType);
-        amount = (int)apply(amount, null, goodsType.getId(), unitType);
+        if (unitType != null) {
+            amount = (int) unitType.apply(amount, null, goodsType.getId(), unitType);
+        } else {
+            /*
+             * XXX: The feature container is always null for TileType. What was
+             *      the desired behaviour here?
+             */
+            amount = (int)apply(amount, null, goodsType.getId(), unitType);
+        }
         return (amount < 0) ? 0 : amount;
     }
 
