@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015   The FreeCol Team
+ *  Copyright (C) 2002-2022   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -24,15 +24,17 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
+import java.util.function.Function;
 import java.util.logging.Level;
 import java.util.logging.Logger;
+import java.util.stream.Collectors;
 
-import javax.xml.stream.XMLStreamConstants;
 import javax.xml.stream.XMLStreamException;
 
 import net.sf.freecol.common.io.FreeColXMLReader;
 import net.sf.freecol.common.io.FreeColXMLWriter;
 import net.sf.freecol.common.model.Colony;
+import static net.sf.freecol.common.model.Constants.*;
 import net.sf.freecol.common.model.FreeColGameObject;
 import net.sf.freecol.common.model.FreeColGameObjectListener;
 import net.sf.freecol.common.model.FreeColObject;
@@ -40,10 +42,11 @@ import net.sf.freecol.common.model.Game;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Specification;
 import net.sf.freecol.common.model.Unit;
+import static net.sf.freecol.common.util.CollectionUtils.*;
+import net.sf.freecol.common.util.LogBuilder;
 import static net.sf.freecol.common.util.RandomUtils.*;
 import static net.sf.freecol.common.util.StringUtils.*;
 import net.sf.freecol.server.FreeColServer;
-import net.sf.freecol.server.model.ServerPlayer;
 
 
 /**
@@ -55,6 +58,8 @@ public class AIMain extends FreeColObject
 
     private static final Logger logger = Logger.getLogger(AIMain.class.getName());
 
+    public static final String TAG = "aiMain";
+
     /** The server that this AI is operating within. */
     private final FreeColServer freeColServer;
 
@@ -62,15 +67,15 @@ public class AIMain extends FreeColObject
     private int nextId = 1;
 
     /**
-     * Contains mappings between <code>FreeColGameObject</code>s
-     * and <code>AIObject</code>s.
+     * Contains mappings between {@code FreeColGameObject}s
+     * and {@code AIObject}s.
      */
     private final Map<String, AIObject> aiObjects = new HashMap<>();
 
 
     /**
-     * Creates a new <code>AIMain</code> and searches the current
-     * game for <code>FreeColGameObject</code>s.
+     * Creates a new {@code AIMain} and searches the current
+     * game for {@code FreeColGameObject}s.
      *
      * @param freeColServer The main controller object for the server.
      */
@@ -79,14 +84,13 @@ public class AIMain extends FreeColObject
     }
 
     /**
-     * Creates a new <code>AIMain</code> and reads the given element.
+     * Creates a new {@code AIMain} from a reader.
      *
      * @param freeColServer The main controller object for the
      *     server.
      * @param xr The input stream containing the XML.
      * @exception XMLStreamException if a problem was encountered
      *     during parsing.
-     * @see #readFromXML
      */
     public AIMain(FreeColServer freeColServer,
                   FreeColXMLReader xr) throws XMLStreamException {
@@ -99,7 +103,7 @@ public class AIMain extends FreeColObject
     /**
      * Gets the main controller object for the server.
      *
-     * @return The <code>FreeColServer</code>-object.
+     * @return The {@code FreeColServer}-object.
      */
     public FreeColServer getFreeColServer() {
         return freeColServer;
@@ -108,19 +112,19 @@ public class AIMain extends FreeColObject
     /**
      * Convenience accessor for the Game.
      *
-     * @return The <code>Game</code> this AI is operating in.
+     * @return The {@code Game} this AI is operating in.
      */
     public Game getGame() {
         return freeColServer.getGame();
     }
 
     /**
-     * Gets a unique identifier for an <code>AIObject</code>.
+     * Gets a unique identifier for an {@code AIObject}.
      *
      * @return A unique identifier.
      */
     public String getNextId() {
-        String id = "am" + Integer.toString(nextId);
+        String id = Integer.toString(nextId);
         nextId++;
         return id;
     }
@@ -138,7 +142,7 @@ public class AIMain extends FreeColObject
     }
 
     /**
-     * Should a <code>FreeColGameObject</code> have a corresponding AI
+     * Should a {@code FreeColGameObject} have a corresponding AI
      * object?
      *
      * Strictly true only for AI players and their units and colonies
@@ -147,7 +151,7 @@ public class AIMain extends FreeColObject
      * which means we can not yet use the Colony or Unit owner fields.
      * So the actual test implemented here is somewhat sloppy.
      *
-     * @param fcgo The <code>FreeColGameObject</code> to test.
+     * @param fcgo The {@code FreeColGameObject} to test.
      * @return True if a corresponding AI object is needed.
      */
     private boolean shouldHaveAIObject(FreeColGameObject fcgo) {
@@ -161,12 +165,12 @@ public class AIMain extends FreeColObject
      * Searches for new {@link FreeColGameObject FreeColGameObjects}.
      * An AI-object is created for each new object.
      *
-     * @param overwrite Determines wether any old <code>AIObject</code>
+     * @param overwrite Determines wether any old {@code AIObject}
      *     should be overwritten or not.
      */
     public void findNewObjects(boolean overwrite) {
         for (FreeColGameObject fcgo : freeColServer.getGame()
-                 .getFreeColGameObjects()) {
+                 .getFreeColGameObjectList()) {
             if (!shouldHaveAIObject(fcgo)) continue;
             if (overwrite || getAIObject(fcgo) == null) {
                 setFreeColGameObject(fcgo.getId(), fcgo);
@@ -175,24 +179,24 @@ public class AIMain extends FreeColObject
     }
 
     /**
-     * Gets the <code>AIObject</code> for the given
-     * <code>FreeColGameObject</code>.
+     * Gets the {@code AIObject} for the given
+     * {@code FreeColGameObject}.
      *
-     * @param fcgo The <code>FreeColGameObject</code> to find the
-     *     <code>AIObject</code> for.
+     * @param fcgo The {@code FreeColGameObject} to find the
+     *     {@code AIObject} for.
      * @see #getAIObject(String)
-     * @return The <code>AIObject</code>.
+     * @return The {@code AIObject}.
      */
-    public AIObject getAIObject(FreeColGameObject fcgo) {
+    private AIObject getAIObject(FreeColGameObject fcgo) {
         return getAIObject(fcgo.getId());
     }
 
     /**
-     * Gets the <code>AIObject</code> for a given object identifier.
+     * Gets the {@code AIObject} for a given object identifier.
      *
      * @param id The object identifier.
      * @see #getAIObject(FreeColGameObject)
-     * @return The <code>AIObject</code>.
+     * @return The {@code AIObject}.
      */
     public AIObject getAIObject(String id) {
         synchronized (aiObjects) {
@@ -201,15 +205,19 @@ public class AIMain extends FreeColObject
     }
 
     /**
-     * Adds a reference to the given <code>AIObject</code>.
+     * Adds a reference to the given {@code AIObject}.
+     *
+     * Note: this is called only for new AI objects, thus it is an
+     * error for the object to already be present in the aiObjects
+     * map.
      *
      * @param id The object identifier.
-     * @param aiObject The <code>AIObject</code> to store a reference
+     * @param aiObject The {@code AIObject} to store a reference
      *        for.
      */
     public void addAIObject(String id, AIObject aiObject) {
         if (aiObject == null) {
-            throw new NullPointerException("aiObject == null");
+            throw new NullPointerException("aiObject == null: " + this);
         }
         boolean present;
         synchronized (aiObjects) {
@@ -222,7 +230,7 @@ public class AIMain extends FreeColObject
     }
 
     /**
-     * Removes a reference to the given <code>AIObject</code>.
+     * Removes a reference to the given {@code AIObject}.
      *
      * @param id The object identifier.
      * @return True if an object for the identifier is removed.
@@ -239,7 +247,7 @@ public class AIMain extends FreeColObject
     /**
      * Get a copy of the list of all AI objects.
      *
-     * @return A list of <code>AIObject</code>s.
+     * @return A list of {@code AIObject}s.
      */
     private List<AIObject> getAIObjects() {
         synchronized (aiObjects) {
@@ -248,12 +256,13 @@ public class AIMain extends FreeColObject
     }
 
     /**
-     * Gets the <code>AIObject</code> with the specified object
+     * Gets the {@code AIObject} with the specified object
      * identifier and class.
      *
+     * @param <T> The actual return type.
      * @param id The object identifier.
      * @param returnClass The expected class of the object.
-     * @return The <code>AIObject</code> found, or null if not.
+     * @return The {@code AIObject} found, or null if not.
      */
     public <T extends AIObject> T getAIObject(String id, Class<T> returnClass) {
         AIObject aio = getAIObject(id);
@@ -267,7 +276,7 @@ public class AIMain extends FreeColObject
     /**
      * Gets the AI colony corresponding to a given colony.
      *
-     * @param colony The <code>Colony</code> to look up.
+     * @param colony The {@code Colony} to look up.
      * @return The corresponding AI colony, or null if not found.
      */
     public AIColony getAIColony(Colony colony) {
@@ -277,7 +286,7 @@ public class AIMain extends FreeColObject
     /**
      * Gets the AI player corresponding to a given player.
      *
-     * @param player The <code>Player</code> to look up.
+     * @param player The {@code Player} to look up.
      * @return The corresponding AI player, or null if not found.
      */
     public AIPlayer getAIPlayer(Player player) {
@@ -287,7 +296,7 @@ public class AIMain extends FreeColObject
     /**
      * Gets the AI unit corresponding to a given unit.
      *
-     * @param unit The <code>Unit</code> to look up.
+     * @param unit The {@code Unit} to look up.
      * @return The corresponding AI unit, or null if not found.
      */
     public AIUnit getAIUnit(Unit unit) {
@@ -297,61 +306,90 @@ public class AIMain extends FreeColObject
     /**
      * Computes how many objects of each class have been created, to
      * track memory leaks over time
+     *
+     * @return A map of AI statistics.
      */
     public Map<String, String> getAIStatistics() {
-        Map<String, String> stats = new HashMap<>();
-        Map<String, Long> objStats = new HashMap<>();
+        Map<String, Long> stats = new HashMap<>();
         for (AIObject aio : getAIObjects()) {
             String className = aio.getClass().getSimpleName();
-            if (objStats.containsKey(className)) {
-                Long count = objStats.get(className);
-                count++;
-                objStats.put(className, count);
+            Long count = stats.get(className);
+            if (count == null) {
+                stats.put(className, 1L);
             } else {
-                Long count = (long) 1;
-                objStats.put(className, count);
+                stats.put(className, count + 1L);
             }
         }
-        for (Entry<String, Long> entry : objStats.entrySet()) {
-            stats.put(entry.getKey(), Long.toString(entry.getValue()));
-        }
-
-        return stats;
+        return transform(stats.entrySet(), alwaysTrue(),
+                         Function.<Entry<String,Long>>identity(),
+                         Collectors.toMap(Entry::getKey,
+                                          e -> Long.toString(e.getValue())));
     }
 
     /**
-     * Checks the integrity of this <code>AIMain</code> by checking if
-     * there are any invalid objects.
+     * Check sort consistency.
      *
-     * @param fix Fix problems if possible.
-     * @return Negative if there are problems remaining, zero if
-     *     problems were fixed, positive if no problems found at all.
+     * @return A string describing sort consistency failures, or null
+     *     on success.
      */
-    public int checkIntegrity(boolean fix) {
-        int result = 1;
+    public String checkSortConsistency() {
+        LogBuilder lb = new LogBuilder(256);
+        lb.add("AI object sort inconsistencies");
+        lb.mark();
+
+        // Check sort consistency of the AI objects
+        AIObject[] objects = getAIObjects().toArray(new AIObject[0]);
+        for (int i0 = 0; i0 < objects.length; i0++) {
+            for (int i1 = i0+1; i1 < objects.length; i1++) {
+                int c0 = objects[i0].compareTo(objects[i1]);
+                int c1 = objects[i1].compareTo(objects[i0]);
+                if (c0 == 0 && c1 == 0) continue;
+                if (c0 == -1 && c1 == 1) continue;
+                if (c0 == 1 && c1 == -1) continue;
+                lb.add("\n  ", objects[i0], " ", objects[i1],
+                       " = ", c0, "/", c1);
+            }
+        }
+        return (lb.grew()) ? lb.toString() : null;
+    }
+    
+    /**
+     * Check the AI integrity too.
+     *
+     * @param fix If true, fix problems if possible.
+     * @param lb A {@code LogBuilder} to log to.
+     * @return -1 if there are problems remaining, zero if problems
+     *     were fixed, +1 if no problems found at all.
+     */
+    public IntegrityType checkIntegrity(boolean fix, LogBuilder lb) {
+        IntegrityType result = IntegrityType.INTEGRITY_GOOD;
         for (AIObject aio : getAIObjects()) {
-            int integ = aio.checkIntegrity(fix);
-            if (integ < 0 && fix) {
-                logger.warning("Invalid AIObject: " + aio.getId()
-                    + " (" + lastPart(aio.getClass().getName(), ".")
-                    + "), dropping.");
-                removeAIObject(aio.getId());
-                aio.dispose();
-                integ = 0;
+            IntegrityType integ = aio.checkIntegrity(fix, lb);
+            if (!integ.safe()) {
+                if (fix) {
+                    lb.add("\n  Invalid AIObject dropped: ", aio.getId(),
+                        "(", lastPart(aio.getClass().getName(), "."), ")");
+                    removeAIObject(aio.getId());
+                    aio.dispose();
+                    result = result.fix();
+                } else {
+                    lb.add("\n  Invalid AIObject: ", aio.getId(),
+                        "(", lastPart(aio.getClass().getName(), "."), ")");
+                    result = result.fail();
+                }
             }
-            result = Math.min(result, integ);
         }
 
-        for (FreeColGameObject fcgo : getGame().getFreeColGameObjects()) {
+        for (FreeColGameObject fcgo : getGame().getFreeColGameObjectList()) {
             if (shouldHaveAIObject(fcgo)
                 && getAIObject(fcgo.getId()) == null) {
                 if (fix) {
-                    logger.warning("Added missing AIObject for: " + fcgo.getId());
+                    lb.add("\n  Missing AIObject added: ", fcgo.getId());
                     setFreeColGameObject(fcgo.getId(), fcgo);
-                    result = 0;
+                    result = result.fix();
                 } else {
-                    logger.warning("Missing AIObject for: " + fcgo.getId());
-                    result = -1;
+                    lb.add("\n  Missing AIObject: ", fcgo.getId());
+                    result = result.fail();
                 }
             }
         }
@@ -362,12 +400,12 @@ public class AIMain extends FreeColObject
     // Interface FreeColGameObjectListener
 
     /**
-     * Creates a new <code>AIObject</code> for a given
-     * <code>FreeColGameObject</code>. This method gets called
+     * Creates a new {@code AIObject} for a given
+     * {@code FreeColGameObject}. This method gets called
      * whenever a new object gets added to the {@link Game}.
      *
      * @param id The object identifier.
-     * @param fcgo The <code>FreeColGameObject</code> to add.
+     * @param fcgo The {@code FreeColGameObject} to add.
      * @see AIObject
      * @see FreeColGameObject
      * @see FreeColGameObject#getId
@@ -376,12 +414,13 @@ public class AIMain extends FreeColObject
     public void setFreeColGameObject(String id, FreeColGameObject fcgo) {
         if (getAIObject(id) != null || !shouldHaveAIObject(fcgo)) return;
         if (!id.equals(fcgo.getId())) {
-            throw new IllegalArgumentException("!id.equals(fcgo.getId())");
+            throw new RuntimeException("!id.equals(fcgo.getId()): " + id);
         }
+        AIObject aio = null;
         if (fcgo instanceof Colony) {
-            new AIColony(this, (Colony)fcgo);
-        } else if (fcgo instanceof ServerPlayer) {
-            ServerPlayer player = (ServerPlayer)fcgo;
+            aio = new AIColony(this, (Colony)fcgo);
+        } else if (fcgo instanceof Player) {
+            Player player = (Player)fcgo;
             if (player.getPlayerType() == null) {
                 // No point doing anything with the object yet, as we
                 // need the player type before we can create the
@@ -389,21 +428,24 @@ public class AIMain extends FreeColObject
                 logger.info("Temporarily ignoring incomplete AI player: "
                     + fcgo.getId());
             } else if (player.isIndian()) {
-                new NativeAIPlayer(this, player);
+                aio = new NativeAIPlayer(this, player);
             } else if (player.isREF()) {
-                new REFAIPlayer(this, player);
+                aio = new REFAIPlayer(this, player);
             } else if (player.isEuropean()) {
-                new EuropeanAIPlayer(this, player);
+                aio = new EuropeanAIPlayer(this, player);
             } else {
-                throw new IllegalArgumentException("Bogus player: " + player);
+                throw new RuntimeException("Bogus player: " + player);
             }
         } else if (fcgo instanceof Unit) {
-            new AIUnit(this, (Unit)fcgo);
+            aio = new AIUnit(this, (Unit)fcgo);
+        }
+        if (aio == null) {
+            throw new RuntimeException("Bogus non-AI FCGO: " + fcgo);
         }
     }
 
     /**
-     * Removes the <code>AIObject</code> for a given AI identifier.
+     * Removes the {@code AIObject} for a given AI identifier.
      * Needed for interface FreeColGameObjectListener.
      *
      * @param id The object identifier.
@@ -418,28 +460,22 @@ public class AIMain extends FreeColObject
     /**
      * Replaces the AI object when ownership changes.
      *
-     * @param source The <code>FreeColGameObject</code> that has changed.
-     * @param oldOwner The old owning <code>Player</code>.
-     * @param newOwner The new owning <code>Player</code>.
+     * @param source The {@code FreeColGameObject} that has changed.
+     * @param oldOwner The old owning {@code Player}.
+     * @param newOwner The new owning {@code Player}.
      */
     @Override
     public void ownerChanged(FreeColGameObject source, Player oldOwner,
                              Player newOwner) {
         AIObject ao = getAIObject(source);
+        if (ao == null) return;
         logger.finest("Owner changed for " + source.getId()
             + " with AI object: " + ao);
-        AIPlayer aiOwner = getAIPlayer(oldOwner);
-        if (aiOwner != null) {
-            if (ao instanceof AIColony) {
-                aiOwner.removeAIColony((AIColony)ao);
-            } else if (ao instanceof AIUnit) {
-                aiOwner.removeAIUnit((AIUnit)ao);
-            }
-        }
-        if (ao != null) {
-            ao.dispose();
-            setFreeColGameObject(source.getId(), source);
-        }
+
+        AIPlayer oldAIOwner = getAIPlayer(oldOwner);
+        if (oldAIOwner != null) oldAIOwner.removeAIObject(ao);
+        ao.dispose();
+        setFreeColGameObject(source.getId(), source);
     }
 
 
@@ -453,18 +489,23 @@ public class AIMain extends FreeColObject
         return getGame().getSpecification();
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public <T extends FreeColObject> boolean copyIn(T other) {
+        AIMain o = copyInCast(other, AIMain.class);
+        if (o == null || !super.copyIn(o)) return false;
+        this.nextId = o.nextId;
+        return true;
+    }
+
 
     // Serialization
 
     private static final String NEXT_ID_TAG = "nextId";
-    // @compat 0.10.3
-    private static final String COLONIAL_AI_PLAYER_TAG = "colonialAIPlayer";
-    private static final String GOODS_WISH_TAG = "GoodsWish";
-    // end @compat
-    // @compat 0.10.7
-    private static final String OLD_NEXT_ID_TAG = "nextID";
-    // end @compat
     // @compat 0.11.3
+    private static final String OLD_GOODS_WISH_TAG = "GoodsWish";
     private static final String OLD_TILE_IMPROVEMENT_PLAN_TAG = "tileimprovementplan";
     // end @compat 0.11.3
 
@@ -487,8 +528,8 @@ public class AIMain extends FreeColObject
     protected void writeChildren(FreeColXMLWriter xw) throws XMLStreamException {
         super.writeChildren(xw);
 
-        for (AIObject aio : FreeColObject.getSortedCopy(aiObjects.values())) {
-            if (aio.checkIntegrity(false) < 0) {
+        for (AIObject aio : sort(getAIObjects())) {
+            if (!aio.checkIntegrity(false).safe()) {
                 // We expect to see integrity failure when AIGoods are
                 // aboard a unit that gets destroyed or if its
                 // destination is destroyed, and probably more.  These
@@ -523,9 +564,6 @@ public class AIMain extends FreeColObject
     @Override
     protected void readAttributes(FreeColXMLReader xr) throws XMLStreamException {
         nextId = xr.getAttribute(NEXT_ID_TAG, -1);
-        // @compat 0.10.x
-        if (nextId < 0) nextId = xr.getAttribute(OLD_NEXT_ID_TAG, 0);
-        // end @compat
     }
 
     /**
@@ -548,94 +586,81 @@ public class AIMain extends FreeColObject
         final String oid = xr.readId();
 
         try {
-            Wish wish = null;
-
             // The AI data is quite shallow, so we can get away with
             // fixing up forward references just with this simple
-            // lookup.  AIObjects that can be forward referenced must
-            // ensure they complete initialization somewhere in their
-            // serialization read* routines.
-            AIObject aio;
+            // lookup.  We complete initialization of such objects
+            // with the call to setInitialized().
+            AIObject aio = null;
+            boolean indirect = false;
             if (oid != null && (aio = getAIObject(oid)) != null) {
                 aio.readFromXML(xr);
+                aio.setInitialized();
 
-            // @compat 0.10.1
-            } else if (COLONIAL_AI_PLAYER_TAG.equals(tag)) {
-                new EuropeanAIPlayer(this, xr);
-            // end @compat
+            } else if (AIColony.TAG.equals(tag)) {
+                aio = new AIColony(this, xr);
 
-            } else if (AIColony.getXMLElementTagName().equals(tag)) {
-                new AIColony(this, xr);
+            } else if (AIGoods.TAG.equals(tag)) {
+                aio = new AIGoods(this, xr);
 
-            } else if (AIGoods.getXMLElementTagName().equals(tag)) {
-                new AIGoods(this, xr);
-
-            } else if (AIPlayer.getXMLElementTagName().equals(tag)) {
+            } else if (AIPlayer.TAG.equals(tag)) {
                 Player p = getGame().getFreeColGameObject(oid, Player.class);
                 if (p != null) {
                     if (p.isIndian()) {
-                        new NativeAIPlayer(this, xr);
+                        aio = new NativeAIPlayer(this, xr);
                     } else if (p.isREF()) {
-                        new REFAIPlayer(this, xr);
+                        aio = new REFAIPlayer(this, xr);
                     } else if (p.isEuropean()) {
-                        new EuropeanAIPlayer(this, xr);
+                        aio = new EuropeanAIPlayer(this, xr);
                     } else {
-                        throw new RuntimeException("Bogus AIPlayer: " + p);
+                        throw new XMLStreamException("Bogus AIPlayer: " + p);
                     }
                 }
 
-            } else if (AIUnit.getXMLElementTagName().equals(tag)) {
-                new AIUnit(this, xr);
+            } else if (AIUnit.TAG.equals(tag)) {
+                aio = new AIUnit(this, xr);
 
-            } else if (GoodsWish.getXMLElementTagName().equals(tag)
-                // @compat 0.10.3
-                || GOODS_WISH_TAG.equals(tag)
-                // end @compat
+            } else if (GoodsWish.TAG.equals(tag)
+                // @compat 0.11.3
+                || OLD_GOODS_WISH_TAG.equals(tag)
+                // end @compat 0.11.3
                        ) {
-                wish = new GoodsWish(this, xr);
+                aio = new GoodsWish(this, xr);
 
-            } else if (TileImprovementPlan.getXMLElementTagName().equals(tag)
-                // @compat 0.10.3
+            } else if (TileImprovementPlan.TAG.equals(tag)
+                // @compat 0.11.3
                 || OLD_TILE_IMPROVEMENT_PLAN_TAG.equals(tag)
                 // end @compat
                        ) {
-                new TileImprovementPlan(this, xr);
+                aio = new TileImprovementPlan(this, xr);
 
-            } else if (WorkerWish.getXMLElementTagName().equals(tag)) {
-                wish = new WorkerWish(this, xr);
+            } else if (WorkerWish.TAG.equals(tag)) {
+                aio = new WorkerWish(this, xr);
             
             } else {
+                System.err.println("AT " + tag + " with " + oid);
+                indirect = true;
                 super.readChild(xr);
             }
-            
-            if (wish != null) {
-                AIColony ac = wish.getDestinationAIColony();
-                if (ac != null) ac.addWish(wish);
+            if (aio == null && !indirect) {
+                throw new XMLStreamException("Bad AI object at " + tag
+                    + " with " + oid);
             }
-
+        } catch (XMLStreamException xse) { // Expected
+            throw xse;
         } catch (Exception e) {
+            // Nothing above apparently tries to throw an exception
+            // that can end up here, but that keeps changing and
+            // this is a great place to resynchronize the reader
+            // so keep this in place.
             logger.log(Level.WARNING, "Exception reading AIObject: "
                        + tag + ", id=" + oid, e);
-            // We are hosed.  Try to resynchronize at the end of the tag
-            // or aiMain.
-            final String mainTag = getXMLElementTagName();
-            while (xr.nextTag() != XMLStreamConstants.END_ELEMENT
-                || !(xr.atTag(tag) || xr.atTag(mainTag)));
+            final String mainTag = TAG;
+            while (xr.moreTags() || !(xr.atTag(tag) || xr.atTag(mainTag)));
         }
     }
 
     /**
      * {@inheritDoc}
      */
-    @Override
-    public String getXMLTagName() { return getXMLElementTagName(); }
-
-    /**
-     * Gets the tag name of the root element representing this object.
-     *
-     * @return "aiMain"
-     */
-    public static String getXMLElementTagName() {
-        return "aiMain";
-    }
+    public String getXMLTagName() { return TAG; }
 }
