@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015   The FreeCol Team
+ *  Copyright (C) 2002-2022   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -21,7 +21,6 @@ package net.sf.freecol.server.ai;
 
 import java.util.ArrayList;
 import java.util.List;
-import java.util.Locale;
 import java.util.logging.Logger;
 
 import javax.xml.stream.XMLStreamException;
@@ -40,17 +39,21 @@ import net.sf.freecol.common.model.Map;
 import net.sf.freecol.common.model.Direction;
 import net.sf.freecol.common.model.PathNode;
 import net.sf.freecol.common.model.Unit;
+import static net.sf.freecol.common.util.CollectionUtils.*;
 import net.sf.freecol.common.util.LogBuilder;
+import static net.sf.freecol.common.util.StringUtils.*;
 
 
 /**
  * An class describing the action needed to make progress in a
  * transportation action for a specific transportable.
  */
-public class Cargo {
+public final class Cargo {
 
     private static final Logger logger = Logger.getLogger(Cargo.class.getName());
 
+    public static final String TAG = "cargo";
+
     /** Abandon cargo after three blockages. */
     private static final int MAX_TRY = 3;
 
@@ -90,9 +93,9 @@ public class Cargo {
         /**
          * Plan the transport of a transportable with a given carrier.
          *
-         * @param t The <code>TransportableAIObject</code> to deliver.
-         * @param carrier The carrier <code>Unit</code> to use.
-         * @param destination The destination <code>Location</code> to take the
+         * @param t The {@code TransportableAIObject} to deliver.
+         * @param carrier The carrier {@code Unit} to use.
+         * @param destination The destination {@code Location} to take the
          *     transportable to, using the transport destination if null.
          * @param allowFallback Allow a fallback plan that does not complete
          *     the transport but does at least improve matters.
@@ -162,7 +165,7 @@ public class Cargo {
             // Where is the transportable dropped?  At the drop node,
             // or at its predecessor from the carrier point of view.
             PathNode drop = pick.getTransportDropNode();
-            if (drop == null || drop.previous == null) {
+            if (drop.previous == null) {
                 throw new IllegalStateException("Cargo failure " + t
                     + " " + deliver.fullPathToString()
                     + " " + pick.fullPathToString()
@@ -219,10 +222,10 @@ public class Cargo {
     /**
      * Create a new cargo.
      *
-     * @param transportable The <code>TransportableAIObject</code>
+     * @param transportable The {@code TransportableAIObject}
      *     to transport.
-     * @param carrier The carrier <code>Unit</code>.
-     * @param plan The <code>CargoPlan</code> to perform.
+     * @param carrier The carrier {@code Unit}.
+     * @param plan The {@code CargoPlan} to perform.
      */
     private Cargo(TransportableAIObject transportable, Unit carrier,
                   CargoPlan plan) {
@@ -237,7 +240,7 @@ public class Cargo {
     /**
      * Create a new cargo from a stream.
      *
-     * @param aiMain The <code>AIMain</code> root.
+     * @param aiMain The {@code AIMain} root.
      * @param xr The input stream containing the XML.
      * @exception XMLStreamException if a problem was encountered
      *     during parsing.
@@ -250,7 +253,7 @@ public class Cargo {
     /**
      * Initialize this cargo.
      *
-     * @param destination The destination <code>Location</code> to take
+     * @param destination The destination {@code Location} to take
      *     the transportable to, defaulting to the transport destination
      *     if null.
      * @param allowFallback Allow a fallback transport.
@@ -274,9 +277,9 @@ public class Cargo {
     /**
      * Make a new cargo with given transportable and carrier.
      *
-     * @param t The <code>TransportableAIObject</code> to transport.
-     * @param carrier The carrier <code>Unit</code> to perform the transport.
-     * @return The new <code>Cargo</code>.
+     * @param t The {@code TransportableAIObject} to transport.
+     * @param carrier The carrier {@code Unit} to perform the transport.
+     * @return The new {@code Cargo}.
      * @exception FreeColException on failure of the planning stage.
      */
     public static Cargo newCargo(TransportableAIObject t, Unit carrier)
@@ -288,12 +291,12 @@ public class Cargo {
      * Make a new cargo with given transportable, carrier and explicit
      * destination and fallback state.
      *
-     * @param t The <code>TransportableAIObject</code> to transport.
-     * @param carrier The carrier <code>Unit</code> to perform the transport.
-     * @param destination The destination <code>Location</code> for
+     * @param t The {@code TransportableAIObject} to transport.
+     * @param carrier The carrier {@code Unit} to perform the transport.
+     * @param destination The destination {@code Location} for
      *     the transportable.
      * @param allowFallback Allow a fallback destination.
-     * @return The new <code>Cargo</code>.
+     * @return The new {@code Cargo}.
      * @exception FreeColException on failure of the planning stage.
      */
     public static Cargo newCargo(TransportableAIObject t, Unit carrier,
@@ -324,7 +327,7 @@ public class Cargo {
     /**
      * Get the transportable.
      *
-     * @return The <code>TransportableAIObject</code> to transport.
+     * @return The {@code TransportableAIObject} to transport.
      */
     public TransportableAIObject getTransportable() {
         return transportable;
@@ -356,8 +359,7 @@ public class Cargo {
 
     public String getModeString() {
         CargoMode mode = getMode();
-        return (mode == null) ? "null"
-            : mode.toString().toLowerCase(Locale.US);
+        return (mode == null) ? "null" : downCase(mode.toString());
     }
 
     public int getTurns() {
@@ -437,7 +439,7 @@ public class Cargo {
     /**
      * Get the movement direction to join the carrier.
      *
-     * @return The <code>Direction</code> to join by.
+     * @return The {@code Direction} to join by.
      */
     public Direction getJoinDirection() {
         return (carrier.isInEurope() || plan.cwait == plan.twait) ? null
@@ -448,7 +450,7 @@ public class Cargo {
     /**
      * Get the movement direction to leave the carrier.
      *
-     * @return The <code>Direction</code> to leave by.
+     * @return The {@code Direction} to leave by.
      */
     public Direction getLeaveDirection() {
         if (!carrier.hasTile() || plan.cdst == plan.tdst) return null;
@@ -469,7 +471,7 @@ public class Cargo {
         ret += (getMode().isCollection()) ? getTransportable().getSpaceTaken()
             : -getTransportable().getSpaceTaken();
         if (hasWrapped()) {
-            ret += wrapped.stream().mapToInt(c -> c.getNewSpace()).sum();
+            ret += sum(wrapped, Cargo::getNewSpace);
         }
         return ret;
     }
@@ -487,7 +489,7 @@ public class Cargo {
      * TransportableAIObjects can be `wrapped' if they have the
      * same target and advancing them reduces the space on the carrier.
      *
-     * @param other The other <code>TransportableAIObject</code>
+     * @param other The other {@code TransportableAIObject}
      *     to consider.
      * @return True if the transportables can be wrapped.
      */
@@ -499,7 +501,7 @@ public class Cargo {
     /**
      * Wrap a Cargo into this one.
      *
-     * @param other The other <code>Cargo</code> to wrap.
+     * @param other The other {@code Cargo} to wrap.
      */
     public void wrap(Cargo other) {
         if (other == this) {
@@ -524,12 +526,12 @@ public class Cargo {
     }
             
     /**
-     * Should this <code>Cargo</code> be retried after encountering
+     * Should this {@code Cargo} be retried after encountering
      * a blockage?  For now, just tries three times.
      *
      * FIXME: be smarter.
      *
-     * @return True if the <code>Cargo</code> should be retried.
+     * @return True if the {@code Cargo} should be retried.
      */
     public boolean retry() {
         return tries++ < MAX_TRY;
@@ -545,7 +547,7 @@ public class Cargo {
     /**
      * Does this cargo involve trade with Europe in a given goods type?
      *
-     * @param type The <code>GoodsType</code> to check.
+     * @param type The {@code GoodsType} to check.
      * @return True if this cargo is of the given type and to be
      *     collected or delivered to Europe.
      */
@@ -558,7 +560,7 @@ public class Cargo {
     /**
      * Check the integrity of this cargo.
      *
-     * @param aiCarrier The <code>AIUnit</code> version of the carrier.
+     * @param aiCarrier The {@code AIUnit} version of the carrier.
      * @return A reason for integrity failure, or null if none.
      */
     public String check(AIUnit aiCarrier) {
@@ -589,15 +591,14 @@ public class Cargo {
      * TODO: be smarter and break out of the loop if the cargo reaches
      * its delivery point.
      *
-     * @param carrier The <code>Unit</code> to queue to.
+     * @param carrier The {@code Unit} to queue to.
      * @param index The queuing position to test.
-     * @param cargoes A list of <code>Cargo</code>s.
+     * @param cargoes A list of {@code Cargo}s.
      * @return True if there is space to add the cargo.
      */
     public boolean canQueueAt(Unit carrier, int index, List<Cargo> cargoes) {
         final int maxHolds = carrier.getCargoCapacity();
         final int newSpace = this.getNewSpace();
-        Cargo tr = cargoes.get(index);
         for (int j = index; j < cargoes.size(); j++) {
             int holds = (j == 0) ? carrier.getCargoSpaceTaken()
                 : maxHolds - cargoes.get(j-1).getSpaceLeft();
@@ -618,36 +619,11 @@ public class Cargo {
         Location lt = getTransportTarget();
         lb.add(" @ ", ((lt == null) ? "null" : lt.toShortString()));
         Location ct = getCarrierTarget();
-        if (ct != lt) lb.add("/", ct.toShortString());
+        if (ct != lt) lb.add("/", ((ct == null) ? "null" : ct.toShortString()));
         return lb.toString();
     }
 
 
-    // Override Object
-
-    /**
-     * {@inheritDoc}
-     */
-    @Override
-    public String toString() {
-        LogBuilder lb = new LogBuilder(64);
-        lb.add("[", transportable,
-            " ", getModeString(),
-            " ", getTurns(), "/", tries, " space=", spaceLeft,
-            ((wrapped == null) ? "" : " wrap"));
-        if (plan.twait != null && plan.cwait != null) {
-            lb.add(" ", plan.twait.toShortString(),
-                "/", plan.cwait.toShortString());
-        }
-        if (plan.cdst != null && plan.tdst != null) {
-            lb.add("->", plan.cdst.toShortString(),
-                "/", plan.tdst.toShortString());
-        }
-        lb.add(" ", plan.fallback, "]");
-        return lb.toString();
-    }            
-
-
     // Serialization
     // Cargo is not yet an AIObject or FreeColObject, but that may happen.
 
@@ -665,7 +641,7 @@ public class Cargo {
 
 
     public void toXML(FreeColXMLWriter xw) throws XMLStreamException {
-        xw.writeStartElement(getXMLElementTagName());
+        xw.writeStartElement(TAG);
 
         xw.writeAttribute(FreeColObject.ID_ATTRIBUTE_TAG,
                           (AIObject)getTransportable());
@@ -709,9 +685,9 @@ public class Cargo {
         if (tid != null) {
             AIObject aio = aiMain.getAIObject(tid);
             if (aio == null) {
-                if (tid.startsWith(Unit.getXMLElementTagName())) {
+                if (tid.startsWith(Unit.TAG)) {
                     tao = new AIUnit(aiMain, tid);
-                } else if (tid.startsWith(AIGoods.getXMLElementTagName())) {
+                } else if (tid.startsWith(AIGoods.TAG)) {
                     tao = new AIGoods(aiMain, tid);
                 }
             } else {
@@ -749,15 +725,36 @@ public class Cargo {
 
         this.plan.fallback = xr.getAttribute(FALLBACK_TAG, false);
 
-        xr.closeTag(getXMLElementTagName());
+        xr.closeTag(TAG);
     }
 
     /**
-     * Gets the tag name of the root element representing this object.
-     *
-     * @return "cargo"
+     * {@inheritDoc}
      */
-    public static String getXMLElementTagName() {
-        return "cargo";
-    }
+    public String getXMLTagName() { return TAG; }
+
+
+    // Override Object
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public String toString() {
+        LogBuilder lb = new LogBuilder(64);
+        lb.add("[", transportable,
+            " ", getModeString(),
+            " ", getTurns(), "/", tries, " space=", spaceLeft,
+            ((wrapped == null) ? "" : " wrap"));
+        if (plan.twait != null && plan.cwait != null) {
+            lb.add(" ", plan.twait.toShortString(),
+                "/", plan.cwait.toShortString());
+        }
+        if (plan.cdst != null && plan.tdst != null) {
+            lb.add("->", plan.cdst.toShortString(),
+                "/", plan.tdst.toShortString());
+        }
+        lb.add(" ", plan.fallback, "]");
+        return lb.toString();
+    }            
 }
