@@ -709,6 +709,15 @@ public class ServerColony extends Colony implements TurnTaker {
             }
         }
 
+        // We are about to process build completions.  When we build
+        // something the production map will be recalculated, so we
+        // need to take a copy now so as to be able to apply the
+        // production from the previous turn, not the new production
+        // with the new buildable present.
+        // See BR#3261 where upgrading the carpenter's shop increases
+        // the lumber consumption.
+        TypeCountMap<GoodsType> productionMap = getProductionMap();
+
         // Check the build queues and build new stuff.  If a queue
         // does a build add it to the built list, so that we can
         // remove the item built from it *after* applying the
@@ -744,14 +753,8 @@ public class ServerColony extends Colony implements TurnTaker {
             }
         }
 
-        // Apply the accumulated production changes.
-        // Beware that if you try to build something requiring hammers
-        // and tools, as soon as one is updated in the colony the
-        // current production cache is invalidated, and the
-        // recalculated one will see the build as incomplete due to
-        // missing the goods just updated.
-        // Hence the need for a copy of the current production map.
-        TypeCountMap<GoodsType> productionMap = getProductionMap();
+        // Apply the accumulated production changes using the saved
+        // production map.
         for (GoodsType goodsType : productionMap.keySet()) {
             int net = productionMap.getCount(goodsType);
             int stored = getGoodsCount(goodsType);
@@ -907,6 +910,11 @@ public class ServerColony extends Colony implements TurnTaker {
                 building.csCheckTeach(teacher, cs);
             }
         }
+        
+        // Repair land units (only used if they have hitpoints).
+        for (Unit unit : transform(getTile().getUnits(), u -> !u.isNaval() && u.isDamaged())) {
+            ((ServerUnit) unit).csRepairUnit(cs);
+        }
 
         // Try to update minimally.
         if (tileDirty) {
