@@ -1,5 +1,5 @@
 /**
- *  Copyright (C) 2002-2015   The FreeCol Team
+ *  Copyright (C) 2002-2022   The FreeCol Team
  *
  *  This file is part of FreeCol.
  *
@@ -28,15 +28,16 @@ import java.awt.Rectangle;
 
 import net.sf.freecol.common.model.Direction;
 import net.sf.freecol.common.model.Game;
-import net.sf.freecol.common.model.GameOptions;
 import net.sf.freecol.common.model.HistoryEvent;
 import net.sf.freecol.common.model.Map;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Region;
 import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.Turn;
-import net.sf.freecol.server.control.ChangeSet;
-import net.sf.freecol.server.control.ChangeSet.See;
+import net.sf.freecol.common.model.Unit;
+import net.sf.freecol.common.networking.ChangeSet;
+import net.sf.freecol.common.networking.ChangeSet.See;
+import net.sf.freecol.common.option.GameOptions;
 import net.sf.freecol.common.util.LogBuilder;
 
 
@@ -58,7 +59,8 @@ public class ServerRegion extends Region {
     /**
      * Trivial constructor for serialization.
      *
-     * @param game The <code>Game</code> to create in.
+     * @param game The {@code Game} to create in.
+     * @param id The object identifier.
      */
     public ServerRegion(Game game, String id) {
         super(game, id);
@@ -66,6 +68,9 @@ public class ServerRegion extends Region {
 
     /**
      * Constructor for copying in a new region from an imported game.
+     *
+     * @param game The {@code Game} to create in.
+     * @param region A {@code Region} to copy.
      */
     public ServerRegion(Game game, Region region) {
         super(game);
@@ -84,8 +89,8 @@ public class ServerRegion extends Region {
     /**
      * Create a new discoverable server region of a given type.
      *
-     * @param game The <code>Game</code> to create in.
-     * @param type The <code>RegionType</code> to use.
+     * @param game The {@code Game} to create in.
+     * @param type The {@code RegionType} to use.
      */
     public ServerRegion(Game game, RegionType type) {
         this(game, null, type, null);
@@ -97,10 +102,10 @@ public class ServerRegion extends Region {
     /**
      * Create a new fixed server region.
      *
-     * @param map The <code>Map</code> to create in.
+     * @param map The {@code Map} to create in.
      * @param key The key for the region.
-     * @param type The <code>RegionType</code> to use.
-     * @param parent The <code>Region</code> to be the parent of this one.
+     * @param type The {@code RegionType} to use.
+     * @param parent The {@code Region} to be the parent of this one.
      */
     private ServerRegion(Map map, String key, RegionType type,
                          Region parent) {
@@ -112,10 +117,10 @@ public class ServerRegion extends Region {
     /**
      * Create a new fixed server region.
      *
-     * @param game The <code>Game</code> to create in.
+     * @param game The {@code Game} to create in.
      * @param key The key for the region.
-     * @param type The <code>RegionType</code> to use.
-     * @param parent The <code>Region</code> to be the parent of this one.
+     * @param type The {@code RegionType} to use.
+     * @param parent The {@code Region} to be the parent of this one.
      */
     private ServerRegion(Game game, String key, RegionType type,
                          Region parent) {
@@ -155,7 +160,7 @@ public class ServerRegion extends Region {
     /**
      * Get the bounding rectangle for this region.
      *
-     * @return The bounding <code>Rectangle</code>.
+     * @return The bounding {@code Rectangle}.
      */
     public final Rectangle getBounds() {
         return bounds;
@@ -173,7 +178,7 @@ public class ServerRegion extends Region {
     /**
      * Get the center of the regions bounds.
      *
-     * @return An two element array [x,y] of the center coordinate.
+     * @return An two member array [x,y] of the center coordinate.
      */
     public int[] getCenter() {
         return new int[] { bounds.x + bounds.width/2,
@@ -183,7 +188,7 @@ public class ServerRegion extends Region {
     /**
      * Does this region contain the center of another?
      *
-     * @param other The other <code>ServerRegion</code> to check.
+     * @param other The other {@code ServerRegion} to check.
      * @return True if the center of the other region is within this one.
      */
     public boolean containsCenter(ServerRegion other) {
@@ -194,7 +199,7 @@ public class ServerRegion extends Region {
     /**
      * Add the given tile to this region.
      *
-     * @param tile A <code>Tile</code> to add.
+     * @param tile A {@code Tile} to add.
      */
     public void addTile(Tile tile) {
         tile.setRegion(this);
@@ -210,38 +215,38 @@ public class ServerRegion extends Region {
     /**
      * Discover this region.
      *
-     * @param player The discovering <code>Player</code>.
-     * @param turn The discovery <code>Turn</code>.
+     * @param player The discovering {@code Player}.
+     * @param unit The discovering {@code Unit}.
+     * @param turn The discovery {@code Turn}.
      * @param newName The name of the region.
-     * @param cs A <code>ChangeSet</code> to update.
+     * @param cs A {@code ChangeSet} to update.
      */
-    public void csDiscover(Player player, Turn turn, String newName,
-                           ChangeSet cs) {
+    public void csDiscover(Player player, Unit unit, Turn turn,
+                           String newName, ChangeSet cs) {
         if (!getDiscoverable()) return;
         final int score = (getSpecification().getBoolean(GameOptions.EXPLORATION_POINTS)) 
             ? this.scoreValue
             : 0;
         if (!hasName()) this.name = newName;
-        for (Region r : discover(player, turn)) cs.add(See.all(), r);
+        for (Region r : discover(player, unit, turn)) cs.add(See.all(), r);
         HistoryEvent h = new HistoryEvent(turn,
             HistoryEvent.HistoryEventType.DISCOVER_REGION, player)
                 .addStringTemplate("%nation%", player.getNationLabel())
                 .addName("%region%", newName);
         h.setScore(score);
         cs.addGlobalHistory(getGame(), h);
-        setDiscoverer(null);
     }
 
     /**
      * Make the fixed regions if they do not exist in a given map.
      *
-     * @param map The <code>Map</code> to check.
-     * @param lb A <code>LogBuilder</code> to log to.
+     * @param map The {@code Map} to check.
+     * @param lb A {@code LogBuilder} to log to.
+     * @return A list of fixed server regions.
      */     
     public static List<ServerRegion> requireFixedRegions(Map map, LogBuilder lb) {
-        final Game game = map.getGame();
         final java.util.Map<String, Region> fixed = map.getFixedRegions();
-        List<ServerRegion> result = new ArrayList<>();
+        List<ServerRegion> result = new ArrayList<>(16);
 
         lb.add("Add regions ");
         // Create arctic/antarctic regions first, but only if they do
@@ -487,7 +492,7 @@ public class ServerRegion extends Region {
                 nNP, " North Pacific, ",
                 nSP, " South Pacific, ",
                 nNA, " North Atlantic, ",
-                nSP, " South Atlantic.\n");
+                nSA, " South Atlantic.\n");
         }
         return result;
     }
@@ -495,10 +500,10 @@ public class ServerRegion extends Region {
     /**
      * Flood fill ocean regions.
      *
-     * @param map The <code>Map</code> to fill in.
-     * @param tile A valid starting <code>Tile</code>.
-     * @param region A <code>ServerRegion</code> to fill with.
-     * @param bounds A <code>Rectangle</code> that bounds the filling.
+     * @param map The {@code Map} to fill in.
+     * @param tile A valid starting {@code Tile}.
+     * @param region A {@code ServerRegion} to fill with.
+     * @param bounds A {@code Rectangle} that bounds the filling.
      * @return The number of tiles filled.
      */
     private static int fillOcean(Map map, Tile tile, ServerRegion region,
@@ -540,8 +545,8 @@ public class ServerRegion extends Region {
         StringBuilder sb = new StringBuilder(32);
         sb.append(super.toString());
         sb.setLength(sb.length() - 1);
-        sb.append(" ").append(size).append(" ").append(bounds)
-            .append("]");
+        sb.append(' ').append(size).append(' ').append(bounds)
+            .append(']');
         return sb.toString();
     }
 }
