@@ -338,7 +338,7 @@ public class ServerUnit extends Unit implements TurnTaker {
         final Player owner =  getOwner();
         setHitPoints(getHitPoints() + 1);
         if (!isDamaged()) {
-            Location loc = getLocation();
+            Location loc = Location.upLoc(getLocation());
             cs.addMessage(owner,
                 new ModelMessage(ModelMessage.MessageType.UNIT_REPAIRED,
                                  "model.unit.unitRepaired",
@@ -999,7 +999,8 @@ public class ServerUnit extends Unit implements TurnTaker {
             && (produce = getWorkType()) != null
             && (learn = spec.getExpertForProducing(produce)) != null
             && learn != getType()
-            && (uc = getUnitChange(UnitChangeType.EXPERIENCE,learn)) != null) {
+            && (uc = getUnitChange(UnitChangeType.EXPERIENCE,learn)) != null
+            && uc.probability > 0) {
             int maximumExperience = getType().getMaximumExperience();
             int maxValue = (100 * maximumExperience) / uc.probability;
             if (maxValue > 0
@@ -1022,7 +1023,7 @@ public class ServerUnit extends Unit implements TurnTaker {
         if (isInMission()) {
             getTile().updateIndianSettlement(owner);
             setMovesLeft(0);
-        } else if (isDamaged()) {
+        } else if (isDamagedAndUnderForcedRepair()) {
             setMovesLeft(0);
         } else {
             setMovesLeft(getInitialMovesLeft());
